"""Map demo showing how to fly to different locations."""

import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.map.types import (
    LatLng,
    ZoomLevelsChangeEvent,
    latlng,
    locate_options,
)

from .common import demo


class FlyToLocationState(rx.State):
    """State class for the fly-to-location map demo."""

    # Define state variables
    zoom: float = 13
    center: LatLng = latlng(lat=51.505, lng=-0.09)
    location_found: LatLng | None = None
    zoom_levels_changed: bool = False

    @rx.event
    def handle_event(self, evt: dict):
        yield rx.toast(
            f"{evt['type']} {evt['latlng']}",
            position="top-center",
        )

    @rx.event
    def handle_zoom_levels_change(self, evt: ZoomLevelsChangeEvent):
        self.zoom_levels_changed = True
        yield rx.toast(
            f"Zoom levels changed! Min zoom: {evt['min_zoom']}, Max zoom: {evt['max_zoom']}",
            position="top-center",
            level="success",
        )

    @rx.event
    def handle_zoom(self, evt: dict):
        self.zoom = round(evt["target"]["zoom"], 4)

    @rx.event
    def handle_location_found(self, latlng_: LatLng):
        """Handle location found event."""
        self.location_found = latlng(lat=latlng_["lat"], lng=latlng_["lng"], nround=4)


pos = latlng(lat=51.505, lng=-0.09)


@demo(
    route="/fly-to-location",
    title="Fly to Location",
    description="Demonstrates flying to different map locations and using geolocation.",
)
def fly_to_location() -> rx.Component:
    map_id = "my-map-foo"
    my_api = rxe.map.api(map_id)
    return rx.vstack(
        rx.hstack(
            rx.heading("Map Demo with Events"),
            rx.text("Zoom: ", FlyToLocationState.zoom),
            rx.text("Located: ", FlyToLocationState.location_found),
        ),
        # Map container with tile layer
        rx.hstack(
            rx.button(
                "Locate",
                on_click=my_api.locate(
                    locate_options(
                        max_zoom=16,
                    )
                ),
            ),
            rx.button(
                "Locate and setView",
                on_click=my_api.locate(
                    locate_options(
                        set_view=True,
                    )
                ),
            ),
            rx.button(
                "Fly to Found Location",
                on_click=rx.cond(
                    FlyToLocationState.location_found,
                    my_api.fly_to(
                        FlyToLocationState.location_found, FlyToLocationState.zoom
                    ),
                    None,
                ),
            ),
            rx.button(
                "Fly to center",
                on_click=my_api.fly_to(
                    FlyToLocationState.center, FlyToLocationState.zoom
                ),
            ),
        ),
        rxe.map(
            # Base tile layer
            rxe.map.tile_layer(
                url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png",
            ),
            rxe.map.marker(
                rxe.map.popup(
                    rx.button("Foo bar", on_click=rx.toast("foo bar from popup"))
                ),
                rxe.map.tooltip("Baz bum"),
                position=pos,
            ),
            rxe.map.marker(
                position=latlng(lat=51.515, lng=-0.1),
                draggable=True,
            ),
            id=map_id,
            zoom=FlyToLocationState.zoom,
            width="100%",
            center=FlyToLocationState.center,
            height="50vh",
            on_click=lambda e: my_api.set_view(
                e.latlng, FlyToLocationState.zoom, {"animate": True}
            ),
            on_zoom=FlyToLocationState.handle_zoom.debounce(100),
            on_locationfound=FlyToLocationState.handle_location_found,
            on_locationerror=rx.toast(
                "Failed to find location",
                description="Check your permissions.",
                level="error",
                position="top-center",
            ),
        ),
        width="100%",
        padding="1em",
    )



"""Constants and configuration for AG Grid components."""

# Version numbers
AG_GRID_VERSION = "33.3.0"
CHARTS_VERSION = "11.2.4"

# Package names
BASE_PKG = "ag-grid-react"
COMMUNITY_PKG = "ag-grid-community"
ENTERPRISE_PKG = "ag-grid-enterprise"
CHARTS_ENTERPRISE_PKG = "ag-charts-enterprise"

# Theme mappings
THEMES = {
    "quartz": {"light": "ag-theme-quartz", "dark": "ag-theme-quartz-dark"},
    "balham": {"light": "ag-theme-balham", "dark": "ag-theme-balham-dark"},
    "alpine": {"light": "ag-theme-alpine", "dark": "ag-theme-alpine-dark"},
    "material": {"light": "ag-theme-material", "dark": "ag-theme-material-dark"},
}
DEFAULT_THEME = "quartz"

# Environment variables
AG_GRID_LICENSE_KEY_ENV = "AG_GRID_LICENSE_KEY"



"""Drag and Drop component for Reflex."""

from __future__ import annotations

from typing import Any, Callable, ClassVar, Mapping, TypedDict, TypeVar

import reflex as rx
from reflex.components.component import Component, MemoizationLeaf
from reflex.components.el.elements import Div
from reflex.constants import Hooks
from reflex.utils.format import to_snake_case
from reflex.vars.base import Var, VarData

from reflex_enterprise.components.component import ComponentEnterprise
from reflex_enterprise.vars import static

# Common type aliases for the DnD components.
JsObj = Mapping[str, Any] | Var[Mapping[str, Any]] | None
ITEM_T = TypeVar("ITEM_T", bound=JsObj)
COLLECT_PARAMS_T = TypeVar(
    "COLLECT_PARAMS_T", bound=rx.vars.ObjectVar[Mapping[str, Any]]
)

# Represents any type of draggable item.
ANY_TYPE = Var.create("Any")
# Placeholder to be replaced with the actual dependency array.
DEP_ARRAY = Var.create(["DEP_ARRAY"])

# Underlying `useDrag` wrapper provided by react-dnd
USE_DRAG = "useDrag"
_use_drag = rx.vars.FunctionStringVar(
    _js_expr=USE_DRAG,
    _var_data=VarData(
        imports={
            "react-dnd": [USE_DRAG],
        },
    ),
)
# Underlying `useDrop` wrapper provided by react-dnd
USE_DROP = "useDrop"
_use_drop = rx.vars.FunctionStringVar(
    _js_expr=USE_DROP,
    _var_data=VarData(
        imports={
            "react-dnd": [USE_DROP],
        },
    ),
)

# Backends for drag and drop
HTML5_BACKEND = "HTML5Backend"
HTML5Backend = Var(
    HTML5_BACKEND,
    _var_data=VarData(
        imports={
            "react-dnd-html5-backend": [HTML5_BACKEND],
        },
    ),
)
# See available options at https://react-dnd.github.io/react-dnd/docs/backends/touch#options
TOUCH_BACKEND = "TouchBackend"
TouchBackend = Var(
    TOUCH_BACKEND,
    _var_data=VarData(
        imports={
            "react-dnd-touch-backend": [TOUCH_BACKEND],
        },
    ),
)


class DnDProvider(ComponentEnterprise):
    """Provides a context for all Drag and Drop operations.

    When using the high-level Draggable and DropTarget components, this will
    automatically be included as an AppWrap component.

    It must be included in the app or page explicitly when using the low-level
    useDrag and useDrop hooks.
    """

    library = "react-dnd"

    tag = "DndProvider"

    backend: Var[str] = HTML5Backend

    # Backend context used to configure the backend
    context: Var[dict[str, Any]]

    # Options object used for configuring the backend
    options: Var[dict[str, Any]]


class BaseDnDDiv(Div, MemoizationLeaf):
    """Base class for DnD components."""

    @staticmethod
    def _get_app_wrap_components() -> dict[tuple[int, str], Component]:
        """Add wrap components for Draggable component."""
        return {
            (30, "DnDProvider"): DnDProvider.create(),
        }


def _get_dep_array(code: Var) -> list[Var]:
    """Get const declarations from useDrag and useDrop hook code.

    When these states change, the dnd hook must be re-evaluated to take into
    account the new values.

    Args:
        code: The code to parse.

    Returns:
        A list of dependencies for the useDrag or useDrop hook.
    """
    deps = []
    var_data = code._get_all_var_data()
    if var_data:
        for hook in var_data.hooks:
            deps.extend(
                Var(d)
                for d in rx.components.component.StatefulComponent._get_hook_deps(hook)
            )
    return deps


def _with_dep_array(code: Var, pattern: str = str(DEP_ARRAY)) -> Var:
    """Update the code to include the dependency array.

    To maximize compatibility, the resulting hook will be marked POST_TRIGGER to
    ensure it can access memoized event triggers.

    Args:
        code: The code to update.
        pattern: The pattern to replace with the dependency array.

    Returns:
        The updated code with the dependency array replaced in for the given pattern.
    """
    dep_array = Var.create(_get_dep_array(code))
    return code._replace(
        _js_expr=str(code).replace(pattern, str(dep_array)),
        merge_var_data=VarData(position=Hooks.HookPosition.POST_TRIGGER),
    )


class MonitorBase(Var):
    """Common methods for DragSourceMonitor and DropTargetMonitor."""

    def get_item_type(self) -> rx.vars.StringVar | Var[None]:
        """Returns a string or a symbol identifying the type of the current dragged item. Returns None if no item is being dragged."""
        return self.getItemType()  # type: ignore[return-value]

    def get_item(self) -> rx.vars.ObjectVar[dict[str, Any]] | Var[None]:
        """Returns a plain object representing the currently dragged item. Returns None if no item is being dragged."""
        return self.getItem()  # type: ignore[return-value]

    def get_drop_result(self) -> rx.Var:
        """Returns a plain object representing the last recorded drop result. Returns None if called outside endDrag()."""
        return self.getDropResult()  # type: ignore[return-value]

    def did_drop(self) -> rx.vars.BooleanVar:
        """Returns True if some drop target has handled the drop event, False otherwise."""
        return self.didDrop()  # type: ignore[return-value]

    def get_initial_client_offset(
        self,
    ) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]:
        """Returns the {x, y} client offset of the pointer at the time when the current drag operation has started. Returns None if no item is being dragged."""
        return self.getInitialClientOffset()  # type: ignore[return-value]

    def get_initial_source_client_offset(
        self,
    ) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]:
        """Returns the {x, y} client offset of the drag source component's root DOM node at the time when the current drag operation has started. Returns None if no item is being dragged."""
        return self.getInitialSourceClientOffset()  # type: ignore[return-value]

    def get_client_offset(self) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]:
        """Returns the last recorded {x, y} client offset of the pointer while a drag operation is in progress. Returns None if no item is being dragged."""
        return self.getClientOffset()  # type: ignore[return-value]

    def get_difference_from_initial_offset(
        self,
    ) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]:
        """Returns the {x, y} difference between the last recorded client offset of the pointer and the client offset when the current drag operation has started. Returns None if no item is being dragged."""
        return self.getDifferenceFromInitialOffset()  # type: ignore[return-value]

    def get_source_client_offset(self) -> rx.vars.ObjectVar[dict[str, int]] | Var[None]:
        """Returns the projected {x, y} client offset of the drag source component's root DOM node, based on its position at the time when the current drag operation has started, and the movement difference. Returns None if no item is being dragged."""
        return self.getSourceClientOffset()  # type: ignore[return-value]

    def __getattr__(self, name: str) -> rx.vars.FunctionVar:
        """Handle corresponding camelCase names as JS functions."""
        # Check to make sure corresponding snake case function exists.
        super().__getattribute__(to_snake_case(name))
        return rx.vars.FunctionVar(
            _js_expr=f"{self._js_expr}.{name}",
            _var_data=self._get_all_var_data(),
        )


class DragSourceMonitor(MonitorBase):
    """Monitor for useDrag hook methods."""

    def can_drag(self) -> rx.vars.BooleanVar:
        """Returns True if no drag operation is in progress, and the owner's canDrag() returns True or is not defined."""
        return self.canDrag()  # type: ignore[return-value]

    def is_dragging(self) -> rx.vars.BooleanVar:
        """Returns True if a drag operation is in progress, and either the owner initiated the drag, or its isDragging() is defined and returns True."""
        return self.isDragging()  # type: ignore[return-value]


class DropTargetMonitor(MonitorBase):
    """Monitor for useDrop hook methods."""

    def can_drop(self) -> rx.vars.BooleanVar:
        """Returns True if there is a drag operation in progress, and the owner's canDrop() returns True or is not defined."""
        return self.canDrop()  # type: ignore[return-value]

    def is_over(
        self,
        options: rx.vars.ObjectVar[dict[str, Any]] = Var.create({}),
    ) -> rx.vars.BooleanVar:
        """Returns True if there is a drag operation in progress, and the owner is the drop target that is currently being hovered.

        Args:
            options: Optional dictionary to strictly check whether only the owner is being hovered (e.g., shallow check).

        Returns:
            BooleanVar indicating whether the drop target is being hovered.
        """
        return self.isOver(options)  # type: ignore[return-value]


def use_drag(
    type: str | Var[str],
    item: ITEM_T | Callable[[DragSourceMonitor], ITEM_T] = None,
    preview_options: JsObj = None,
    options: JsObj = None,
    end: Callable[[ITEM_T, DragSourceMonitor], None]
    | Var[Callable[[ITEM_T, DragSourceMonitor], None]]
    | None = None,
    can_drag: Callable[[DragSourceMonitor], bool | Var[bool]]
    | Var[Callable[[DragSourceMonitor], bool | Var[bool]]]
    | None = None,
    is_dragging: Callable[[DragSourceMonitor], bool | Var[bool]]
    | Var[Callable[[DragSourceMonitor], bool | Var[bool]]]
    | None = None,
    collect: Callable[[DragSourceMonitor], COLLECT_PARAMS_T]
    | Var[Callable[[DragSourceMonitor], COLLECT_PARAMS_T]]
    | None = None,
    _collected_params: Var[COLLECT_PARAMS_T] | None = None,
    _drag_ref: Var | None = None,
    _drag_preview_ref: Var | None = None,
) -> tuple[
    COLLECT_PARAMS_T,  # collected params
    rx.Var,  # drag ref
    rx.Var,  # drag preview ref
]:
    """Wrapper for the useDrag hook, for advanced use cases.

    Note the functions passed to this hook are NOT event handlers in state, they
    must be provided as LambdaVar-able functions that accept Var-type arguments
    and return Var-type values. To use event handlers instead, consider using
    the high-level APIs `on_end` provided by the `Draggable` component.

    To make an element into a drag source, assign the returned drag_ref as the
    `ref` of some component:

    ```python
        collected_params, drag_ref, drag_preview_ref = use_drag(...)
        return rx.el.div(
            rx.text("Drag me"),
            custom_attrs={"ref": drag_ref},
        )
    ```

    The returned `collected_params` can be used to access the value returned by
    the `collect` function while rendering the component.

    For example, to access the `is_dragging` value:

    ```python
        collected_params, drag_ref, drag_preview_ref = use_drag(
            collect=lambda monitor: {
                "is_dragging": monitor.is_dragging(),
            }
        )
        return rx.el.div(
            rx.text("Drag me"),
            custom_attrs={"ref": drag_ref},
            opacity=rx.cond(collected_params.is_dragging, 0.5, 1),
        )
    ```

    Args:
        type: The type of the item to be dragged.
        item: The item representation, recommended to simply include an identifier.
        preview_options: Options for the drag preview.
        options: Options for the drag source ("dropEffect").
        end: Function to be called when the drag operation ends.
        can_drag: Function to determine if the item can be dragged.
        is_dragging: Function to override if a drag is in progress.
        collect: Function to collect properties to be accessed in the rendered component.
        _collected_params: Use this var for collected params instead of auto generating.
        _drag_ref: Use this var for the drag ref instead of auto generating.
        _drag_preview_ref: Use this var for the drag preview ref instead of auto generating.

    Returns:
        A tuple containing the collected params, drag ref, and drag preview ref.
    """
    spec: dict[str, Var] = {"type": Var.create(type)}
    if item is not None:
        spec["item"] = Var.create(item)
    if preview_options is not None:
        spec["previewOptions"] = Var.create(preview_options)
    if options is not None:
        spec["options"] = Var.create(options)
    if end is not None:
        spec["end"] = Var.create(end)
    if can_drag is not None:
        spec["canDrag"] = Var.create(can_drag)
    if is_dragging is not None:
        spec["isDragging"] = Var.create(is_dragging)
    if collect is not None:
        spec["collect"] = Var.create(collect)

    collected_params = (
        _collected_params
        if _collected_params is not None
        else rx.vars.ObjectVar(
            f"collected_params_{rx.vars.get_unique_variable_name()}",
            _var_type=Mapping[str, Any],
        )
    )
    drag_ref = (
        _drag_ref
        if _drag_ref is not None
        else Var(f"drag_ref_{rx.vars.get_unique_variable_name()}")
    )
    drag_preview_ref = (
        _drag_preview_ref
        if _drag_preview_ref is not None
        else Var(f"drag_preview_ref_{rx.vars.get_unique_variable_name()}")
    )
    code = _with_dep_array(
        Var(
            f"const [{collected_params}, {drag_ref}, {drag_preview_ref}] = "
            f"{_use_drag(Var.create(spec), DEP_ARRAY)};"
        )
    )
    _var_data = VarData.merge(
        code._get_all_var_data(),  # merge hook dependencies first for < 0.7.11 compat
        VarData(
            hooks={str(code): code._get_all_var_data()},
            position=Hooks.HookPosition.POST_TRIGGER,
        ),
    )
    return (
        collected_params._replace(merge_var_data=_var_data),  # type: ignore[reportReturnType]
        drag_ref._replace(merge_var_data=_var_data),
        drag_preview_ref._replace(merge_var_data=_var_data),
    )


class Draggable(BaseDnDDiv):
    """Draggable component wrapper around use_drag."""

    # Type of the draggable item. Checked by the drop target `accept` prop.
    type: Var[str]

    # The item representations, recommended to simply include an identifier.
    item: Var[JsObj | Callable[[DragSourceMonitor], JsObj]]
    # Options for the drag preview.
    preview_options: Var[JsObj]
    # Options for the drag source ("dropEffect").
    options: Var[JsObj]

    # An event handler to call when the drag operation ends.
    on_end: rx.EventHandler[rx.event.passthrough_event_spec(dict[str, Any])]

    # These are used to attach the hook to the DOM node.
    drag_ref: Var[Any]
    drag_preview_ref: Var[Any]

    # NOTE: These callables are NOT event handlers in state, they must be LambdaVar
    # Function to be called when the drag operation ends (supersedes on_end).
    end: Var[Callable[[JsObj, DragSourceMonitor], None]]
    # Predicate function to determine if the item can be dragged.
    can_drag: Var[Callable[[DragSourceMonitor], bool | Var[bool]]]
    # Predicate function to override if a drag is in progress.
    is_dragging: Var[Callable[[DragSourceMonitor], bool | Var[bool]]]
    # Function to collect properties to be accessed in the rendered component (prefer subclassing and overriding `collect_hook`).
    collect: Var[Callable[[DragSourceMonitor], JsObj]]

    def _exclude_props(self) -> list[str]:
        """Exclude props from being passed to the component."""
        return ["drag_ref", "drag_preview_ref", "on_end"]

    @classmethod
    def create(cls, *children, **props) -> Component:
        """Create a Draggable component.

        Will use the `collect_hook` to collect the params if
        not provided, which is represented by the `Draggable.CollectedParams`
        type and accessed via `Draggable.collected_params`.
        """
        if "on_end" in props:
            props.setdefault(
                "end",
                rx.EventChain.create(
                    props["on_end"],
                    args_spec=rx.event.passthrough_event_spec(dict[str, Any]),
                    key="on_end",
                ),
            )
        (
            params,
            props["drag_ref"],
            props["drag_preview_ref"],
        ) = use_drag(
            type=props.pop("type", ANY_TYPE),
            item=props.pop("item", None),
            preview_options=props.pop("preview_options", None),
            options=props.pop("options", None),
            end=props.pop("end", None),
            can_drag=props.pop("can_drag", None),
            is_dragging=props.pop("is_dragging", None),
            collect=props.pop(
                "collect",
                Var.create(
                    cls.collect_hook,
                ),
            ),
            _collected_params=props.pop(
                "_collected_params",
                cls.collected_params,
            ),
            _drag_ref=props.pop("drag_ref", None),
            _drag_preview_ref=props.pop("drag_preview_ref", None),
        )
        props.setdefault("custom_attrs", {})["ref"] = props["drag_ref"]

        return super().create(
            *children,
            **props,
            opacity=rx.cond(params.is_dragging, 0.5, 1),
        )

    class CollectedParams(TypedDict):
        """Collected params for the Draggable component."""

        is_dragging: bool
        can_drag: bool

    @staticmethod
    @static
    def collect_hook(
        monitor: DragSourceMonitor,
    ) -> rx.Var[Draggable.CollectedParams]:
        """Default collect hook for the Draggable component.

        Args:
            monitor: The drag source monitor.

        Returns:
            A dictionary with the collected properties.
        """
        return rx.Var.create(
            {
                "is_dragging": monitor.is_dragging().bool(),
                "can_drag": monitor.can_drag(),
            },
        ).to(Draggable.CollectedParams)

    # For accessing the default collected params for the Draggable component.
    collected_params: ClassVar[rx.vars.ObjectVar[Draggable.CollectedParams]] = (
        rx.vars.ObjectVar("draggableCollectedParams", _var_type=CollectedParams)
    )


def use_drop(
    accept: str | Var[str],
    options: JsObj = None,
    drop: Callable[[dict[str, Any], DropTargetMonitor], None]
    | Var[Callable[[dict[str, Any], DropTargetMonitor], None]]
    | None = None,
    hover: Callable[[dict[str, Any], DropTargetMonitor], None]
    | Var[Callable[[dict[str, Any], DropTargetMonitor], None]]
    | None = None,
    can_drop: Callable[[DropTargetMonitor], bool | Var[bool]]
    | Var[Callable[[DropTargetMonitor], bool | Var[bool]]]
    | None = None,
    collect: Callable[[DropTargetMonitor], COLLECT_PARAMS_T]
    | Var[Callable[[DropTargetMonitor], COLLECT_PARAMS_T]]
    | None = None,
    _collected_params: COLLECT_PARAMS_T | None = None,
    _drop_ref: Var | None = None,
) -> tuple[
    COLLECT_PARAMS_T,  # collected params
    rx.Var,  # drop ref
]:
    """Wrapper for the useDrop hook, for advanced use cases.

    Note the functions passed to this hook are NOT event handlers in state, they
    must be provided as LambdaVar-able functions that accept Var-type arguments
    and return Var-type values. To use event handlers instead, consider using
    the high-level APIs `on_drop` and `on_hover` provided by the `DropTarget`
    component.

    To make an element into a drop target, assign the returned drop_ref as the
    `ref` of some component:

    ```python
        collected_params, drop_ref = use_drop(...)
        return rx.el.div(
            rx.text("Drop here"),
            custom_attrs={"ref": drop_ref},
        )
    ```

    The returned `collected_params` can be used to access the value returned by
    the `collect` function while rendering the component.

    For example, to access the `is_over` value:

    ```python
        collected_params, drop_ref = use_drop(
            collect=lambda monitor: {
                "is_over": monitor.is_over(),
            }
        )
        return rx.el.div(
            rx.text("Drop here"),
            custom_attrs={"ref": drop_ref},
            background_color=rx.cond(collected_params.is_over, "green", "blue"),
        )
    ```

    Args:
        accept: The type of the item to accept.
        options: Options for the drop target.
        drop: Function to be called when an item is dropped.
        hover: Function to be called when an item is hovered.
        can_drop: Function to determine if the item can be dropped.
        collect: Function to collect properties to be accessed in the rendered component.
        _collected_params: Use this var for collected params instead of auto generating.
        _drop_ref: Use this var for the drop ref instead of auto generating.

    Returns:
        A tuple containing the collected params and the drop ref.
    """
    spec: dict[str, Var] = {"accept": Var.create(accept)}
    if options is not None:
        spec["options"] = Var.create(options)
    if drop is not None:
        spec["drop"] = Var.create(drop)
    if hover is not None:
        spec["hover"] = Var.create(hover)
    if can_drop is not None:
        spec["canDrop"] = Var.create(can_drop)
    if collect is not None:
        spec["collect"] = Var.create(collect)

    collected_params = (
        _collected_params
        if _collected_params is not None
        else rx.vars.ObjectVar(
            f"collected_params_{rx.vars.get_unique_variable_name()}",
            _var_type=Mapping[str, Any],
        )
    )
    drop_ref = (
        _drop_ref
        if _drop_ref is not None
        else Var(f"drop_ref_{rx.vars.get_unique_variable_name()}")
    )
    code = _with_dep_array(
        Var(
            f"const [{collected_params}, {drop_ref}] = "
            f"{_use_drop(Var.create(spec), DEP_ARRAY)};"
        )
    )
    _var_data = VarData.merge(
        code._get_all_var_data(),  # merge hook dependencies first for < 0.7.11 compat
        VarData(
            hooks={str(code): code._get_all_var_data()},
            position=Hooks.HookPosition.POST_TRIGGER,
        ),
    )
    return (
        collected_params._replace(merge_var_data=_var_data),  # type: ignore[reportReturnType]
        drop_ref._replace(merge_var_data=_var_data),
    )


class DropTarget(BaseDnDDiv):
    """Droppable component wrapper around use_drop."""

    # Acceptable types for the drop target
    accept: Var[str | list[str]]

    # Options for the drop target.
    options: Var[JsObj]

    # Event handler to be called when an item is dropped.
    on_drop: rx.EventHandler[rx.event.passthrough_event_spec(dict[str, Any])]

    # Event handler to be called when an item is hovered.
    on_hover: rx.EventHandler[rx.event.passthrough_event_spec(dict[str, Any])]

    # These are used to attach the hook to the DOM node, override the auto-generated value.
    drop_ref: Var[Any]

    # NOTE: These callables are NOT event handlers in state, they must be LambdaVar
    # Advanced: Function to be called when an item is dropped (supersedes on_drop)
    drop: Var[Callable[[dict[str, Any], DropTargetMonitor], None]]
    # Advanced: Function to be called when an item is hovered (supersedes on_hover)
    hover: Var[Callable[[dict[str, Any], DropTargetMonitor], None]]
    # Function to determine if the item can be dropped (must return bool).
    can_drop: Var[Callable[[dict[str, Any], DropTargetMonitor], bool | Var[bool]]]
    # Function to collect properties to be accessed in the rendered component (prefer subclassing and overriding `collect_hook`).
    collect: Var[Callable[[DropTargetMonitor], JsObj]]

    def _exclude_props(self) -> list[str]:
        """Exclude props from being passed to the component."""
        return ["drop_ref", "on_drop", "on_hover"]

    @classmethod
    def create(cls, *children, **props) -> Component:
        """Create a Droppable component.

        Will use the class `collect_hook` to collect the params if
        not provided, which is represented by the `DropTarget.CollectedParams`
        type and accessed via `DropTarget.collected_params` var.
        """
        if "on_drop" in props:
            props.setdefault(
                "drop",
                rx.EventChain.create(
                    props["on_drop"],
                    args_spec=rx.event.passthrough_event_spec(dict[str, Any]),
                    key="on_drop",
                ),
            )
        if "on_hover" in props:
            props.setdefault(
                "hover",
                rx.EventChain.create(
                    props["on_hover"],
                    args_spec=rx.event.passthrough_event_spec(dict[str, Any]),
                    key="on_hover",
                ),
            )
        (
            _,
            props["drop_ref"],
        ) = use_drop(
            accept=props.pop("accept", ANY_TYPE),
            options=props.pop("options", None),
            drop=props.pop("drop", None),
            hover=props.pop("hover", None),
            can_drop=props.pop("can_drop", None),
            collect=props.pop(
                "collect",
                Var.create(
                    cls.collect_hook,
                ),
            ),
            _collected_params=props.pop(
                "_collected_params",
                cls.collected_params,
            ),
            _drop_ref=props.pop("drop_ref", None),
        )
        props.setdefault("custom_attrs", {})["ref"] = props["drop_ref"]
        return super().create(
            *children,
            **props,
        )

    class CollectedParams(TypedDict):
        """Collected params for the DropTarget component."""

        is_over: bool
        can_drop: bool
        item: dict[str, Any] | None

    @staticmethod
    @static
    def collect_hook(
        monitor: DropTargetMonitor,
    ) -> rx.Var[DropTarget.CollectedParams]:
        """Default collect hook for the DropTarget component.

        Args:
            monitor: The drop target monitor.

        Returns:
            A dictionary with the collected properties.
        """
        return rx.Var.create(
            {
                "is_over": monitor.is_over(),
                "can_drop": monitor.can_drop(),
                "item": monitor.get_item(),
            },
        ).to(DropTarget.CollectedParams)

    # For accessing the default collected params for the DropTarget component.
    collected_params: ClassVar[rx.vars.ObjectVar[DropTarget.CollectedParams]] = (
        rx.vars.ObjectVar("dropTargetCollectedParams", _var_type=CollectedParams)
    )


class DnDNamespace(rx.ComponentNamespace):
    """Namespace for DnD components."""

    HTML5Backend: ... = HTML5Backend
    TouchBackend: ... = TouchBackend
    provider = staticmethod(DnDProvider.create)

    Draggable: type[Draggable] = Draggable
    draggable = staticmethod(Draggable.create)
    use_drag = staticmethod(use_drag)
    DragSourceMonitor: type[DragSourceMonitor] = DragSourceMonitor

    DropTarget: type[DropTarget] = DropTarget
    drop_target = staticmethod(DropTarget.create)
    use_drop = staticmethod(use_drop)
    DropTargetMonitor: type[DropTargetMonitor] = DropTargetMonitor


dnd: DnDNamespace = DnDNamespace()



"""Stub file for reflex_enterprise/components/mantine/tags_input.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from typing import Any, Callable, Literal, Mapping, Optional, Sequence

from reflex.components.component import Component
from reflex.components.core.breakpoints import Breakpoints
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import BaseMantineInput

class TagsInput(BaseMantineInput):
    @classmethod
    def create(
        cls,
        *children,
        accept_value_on_blur: Var[bool] | bool | None = None,
        allow_duplicates: Var[bool] | bool | None = None,
        clear_button_props: Var[dict] | dict | None = None,
        combobox_props: Var[dict] | dict | None = None,
        data: Var[list[str]] | list[str] | None = None,
        default_dropdown_opened: Var[bool] | bool | None = None,
        default_search_value: Var[str] | str | None = None,
        default_value: Var[list[str]] | list[str] | None = None,
        description: Component | Var[Component | str] | str | None = None,
        description_props: Var[dict] | dict | None = None,
        disabled: Var[bool] | bool | None = None,
        dropdown_opened: Var[bool] | bool | None = None,
        error: Component | Var[Component | bool | str] | bool | str | None = None,
        error_props: Var[dict] | dict | None = None,
        filter: Callable | Var[Callable] | None = None,
        hidden_input_props: Var[dict] | dict | None = None,
        hidden_input_values_divider: Var[str] | str | None = None,
        input_container: Component | Var[Component] | None = None,
        input_size: Var[str] | str | None = None,
        input_wrapper_order: Var[
            list[Literal["description", "error", "input", "label"]]
        ]
        | list[Literal["description", "error", "input", "label"]]
        | None = None,
        label: Component | Var[Component | str] | str | None = None,
        label_props: Var[dict] | dict | None = None,
        limit: Var[int] | int | None = None,
        max_dropdown_height: Var[float | str] | float | str | None = None,
        max_tags: Var[int] | int | None = None,
        pointer: Var[bool] | bool | None = None,
        radius: Literal["lg", "md", "sm", "xl", "xs"]
        | Var[Literal["lg", "md", "sm", "xl", "xs"] | int | str]
        | int
        | str
        | None = None,
        render_option: Callable | Var[Callable] | None = None,
        required: Var[bool] | bool | None = None,
        scroll_area_props: Var[dict] | dict | None = None,
        search_value: Var[str] | str | None = None,
        select_first_option_on_change: Var[bool] | bool | None = None,
        size: Literal["lg", "md", "sm", "xl", "xs"]
        | Var[Literal["lg", "md", "sm", "xl", "xs"]]
        | None = None,
        split_chars: Var[list[str]] | list[str] | None = None,
        value: Var[list[str]] | list[str] | None = None,
        with_asterisk: Var[bool] | bool | None = None,
        with_error_styles: Var[bool] | bool | None = None,
        with_scroll_area: Var[bool] | bool | None = None,
        clearable: Var[bool] | bool | None = None,
        placeholder: Var[str] | str | None = None,
        multiline: Var[bool] | bool | None = None,
        read_only: Var[bool] | bool | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        left_section: Component | Var[Component | str] | str | None = None,
        left_section_props: Var[dict] | dict | None = None,
        left_section_width: Var[float | str] | float | str | None = None,
        left_section_pointer_events: Var[str] | str | None = None,
        right_section: Component | Var[Component | str] | str | None = None,
        right_section_props: Var[dict] | dict | None = None,
        right_section_width: Var[float | str] | float | str | None = None,
        right_section_pointer_events: Var[str] | str | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_change: Optional[EventType[()] | EventType[list[str]]] = None,
        on_clear: Optional[EventType[Any]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_dropdown_close: Optional[EventType[Any]] = None,
        on_dropdown_open: Optional[EventType[Any]] = None,
        on_duplicate: Optional[EventType[()] | EventType[str]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_option_submit: Optional[EventType[()] | EventType[str]] = None,
        on_remove: Optional[EventType[()] | EventType[str]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_search_change: Optional[EventType[Any]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "TagsInput":
        """Create the Reflex Enterprise component."""
        ...

tags_input = TagsInput.create



"""Cell selection demo for AG Grid in Reflex."""

import pandas as pd
import reflex as rx

import reflex_enterprise as rxe

from .common import demo


class CellSelectionState(rx.State):
    """State for the cell selection demo."""

    data: list[dict] = []

    @rx.event
    def load_data(self):
        """Load data into the state."""
        df = pd.read_json("https://www.ag-grid.com/example-assets/olympic-winners.json")
        self.data = df.to_dict("records")

    @rx.event
    def echo_selection(self, ranges: list[dict], started: bool, finished: bool):
        """Echo the selected cells."""
        if finished:
            yield rx.toast(f"Selected cells: {ranges}")


column_defs = [
    {"field": "athlete"},
    {"field": "age"},
    {"field": "country"},
    {"field": "year"},
    {"field": "sport"},
    {
        "header_name": "Medals",
        "children": [
            {
                "field": "total",
                "column_group_show": "closed",
                "col_id": "total",
                "value_getter": "params.data.gold + params.data.silver + params.data.bronze",
                "width": 100,
            },
            {"field": "gold", "column_group_show": "open", "width": 100},
            {"field": "silver", "column_group_show": "open", "width": 100},
            {"field": "bronze", "column_group_show": "open", "width": 100},
        ],
    },
]


@demo(
    route="/cell-selection",
    title="Cell Selection",
    description="Demonstrates cell selection in AG Grid.",
    on_load=CellSelectionState.load_data,
)
def cell_selection_page():
    """Cell selection demo."""
    return rxe.ag_grid(
        id="cell_selection_grid",
        column_defs=column_defs,
        row_data=CellSelectionState.data,
        cell_selection=True,
        on_cell_selection_changed=CellSelectionState.echo_selection,
        width="100%",
        height="600px",
    )



"""Kanban Drag n Drop."""

import dataclasses
import json
import uuid
from typing import Any, Callable, Mapping

import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.dnd import DropTargetMonitor

from .common import demo

COL_MIN_WIDTH = "300px"
COL_HEIGHT = "80vh"


@dataclasses.dataclass
class KanbanColumn:
    """Kanban column."""

    __drag_type__ = "KanbanDraggableColumn"

    id: str
    title: str
    description: str
    order: int


@dataclasses.dataclass
class ItemInfo:
    """Item information."""

    __drag_type__ = "KanbanDraggableItemInfo"

    id: str
    title: str
    description: str = ""
    order: int = 0
    column_id: str = ""


UnknownItem = ItemInfo(id="unknown", title="Error")


class KanbanState(rx.State):
    _columns: dict[str, KanbanColumn] = {}
    _items: dict[str, ItemInfo] = {}

    kanban_data_json: str = rx.LocalStorage()

    @rx.event
    def on_load(self):
        """Load the kanban data from local storage."""
        if self.kanban_data_json:
            try:
                data = json.loads(self.kanban_data_json)
            except ValueError:
                self.kanban_data_json = ""
                return rx.toast("Could not load your data from browser storage, sorry.")

            for col_data in data.get("columns", []):
                col = KanbanColumn(**col_data)
                self._columns[col.id] = col

            for item_data in data.get("items", []):
                item = ItemInfo(**item_data)
                self._items[item.id] = item

    @rx.event
    def on_save(self):
        """Save the kanban data to local storage."""
        data = {
            "columns": [dataclasses.asdict(col) for col in self._columns.values()],
            "items": [dataclasses.asdict(item) for item in self._items.values()],
        }
        self.kanban_data_json = json.dumps(data)

    @rx.var
    def items(self) -> dict[str, ItemInfo]:
        """Get items."""
        return self._items

    @rx.var
    def ordered_items_by_column_id(self) -> dict[str, list[ItemInfo]]:
        """Get items ordered by column."""
        items_by_column = {}
        for item in self._items.values():
            items_by_column.setdefault(item.column_id, []).append(item)
        return {
            col_id: sorted(items, key=lambda x: x.order)
            for col_id, items in items_by_column.items()
        }

    @rx.var
    def columns(self) -> dict[str, KanbanColumn]:
        """Get columns."""
        return self._columns

    @rx.var
    def ordered_columns(self) -> list[KanbanColumn]:
        """Get columns ordered by order."""
        columns = list(self._columns.values())
        columns.sort(key=lambda x: x.order)
        return columns

    @rx.event
    def on_item_drop(
        self,
        column_id: str,
        replace_position: int,
        dropped_item_data: dict[str, Any],
    ):
        """Handle item drop."""
        dropped_item_id = dropped_item_data.get("id")
        if not dropped_item_id:
            return
        dropped_item = self._items.get(dropped_item_id)
        if not dropped_item:
            return
        new_items = self.ordered_items_by_column_id.get(column_id, [])
        try:
            ix_prior_to_drop = new_items.index(dropped_item)
            # When the item is moving to a higher position in the same column,
            # decrement the replace_position to account for the item being removed
            if ix_prior_to_drop < replace_position:
                replace_position -= 1
        except ValueError:
            # The item is not in the new column, so we don't need to adjust the position
            pass
        dropped_item.column_id = column_id
        new_items = [
            item
            for item in self.ordered_items_by_column_id.get(column_id, [])
            if item.id != dropped_item.id
        ]
        if replace_position < 0:
            new_items.append(dropped_item)
        else:
            new_items.insert(replace_position, dropped_item)
        # Update the order for all items in the column
        for ix, item in enumerate(new_items):
            item.order = ix
            self._items[item.id] = item
        self.on_save()

    @rx.event
    def on_column_drop(
        self,
        replace_position: int,
        dropped_column_data: dict[str, Any],
    ):
        """Handle column drop."""
        dropped_column_id = dropped_column_data.get("id")
        if not dropped_column_id:
            return
        dropped_column = self._columns.get(dropped_column_id)
        if not dropped_column:
            return
        new_columns = self.ordered_columns
        try:
            ix_prior_to_drop = new_columns.index(dropped_column)
            # When the column is moving to a higher position, decrement the
            # replace_position to account for the column being removed
            if ix_prior_to_drop < replace_position:
                replace_position -= 1
        except ValueError:
            pass
        new_columns.remove(dropped_column)
        if replace_position < 0:
            new_columns.append(dropped_column)
        else:
            new_columns.insert(replace_position, dropped_column)
        # Update the order for all columns
        for ix, col in enumerate(new_columns):
            col.order = ix
            self._columns[col.id] = col
        self.on_save()

    @rx.event
    def new_column(self, form_data: dict[str, Any]):
        """Create a new column."""
        name = form_data.get("name")
        if not name:
            return
        col = KanbanColumn(
            id=str(uuid.uuid4()),
            title=name,
            description="",
            order=len(self._columns),
        )
        self._columns[col.id] = col
        self.on_save()

    @rx.event
    def new_item(self, form_data: dict[str, Any]):
        """Create a new item."""
        column_id = form_data.get("column_id")
        name = form_data.get("name")
        if not column_id or not name:
            return
        item = ItemInfo(
            id=str(uuid.uuid4()),
            title=name,
            description="",
            order=len(self.ordered_items_by_column_id.get(column_id, [])),
            column_id=column_id,
        )
        self._items[item.id] = item
        self.on_save()

    @classmethod
    def can_drop_item(
        cls,
        column_id: rx.vars.StringVar[str] | str,
        replace_position: rx.vars.NumberVar[int] | int,
    ) -> Callable[[rx.Var[Any], DropTargetMonitor], rx.Var[bool]]:
        n_items_in_column = KanbanState.ordered_items_by_column_id.get(
            column_id, []
        ).length()

        @rxe.static
        def _can_drop(
            item: rx.vars.ObjectVar[Mapping[str, str]], monitor: DropTargetMonitor
        ) -> rx.Var[bool]:
            dragged_item = cls.item_by_id(item.id)
            return (dragged_item.column_id != column_id) | ~rx.Var.create(
                [dragged_item.order, dragged_item.order.to(int) + 1]
            ).contains(
                rx.cond(
                    replace_position < 0,
                    n_items_in_column,
                    replace_position,
                )
            )

        return _can_drop

    @classmethod
    def item_by_id(cls, item_id: rx.Var[str]) -> rx.vars.ObjectVar[ItemInfo]:
        """Get an item by its ID."""
        return cls.items.get(item_id.to(str), UnknownItem).to(ItemInfo)

    @classmethod
    def can_drop_column(
        cls,
        replace_position: rx.vars.NumberVar[int] | int,
    ) -> Callable[[rx.Var[Any], DropTargetMonitor], rx.Var[bool]]:
        n_columns = KanbanState.ordered_columns.length()

        @rxe.static
        def _can_drop(
            item: rx.vars.ObjectVar[Mapping[str, str]], monitor: DropTargetMonitor
        ) -> rx.Var[bool]:
            dragged_column = cls.columns[item.id]
            return ~rx.Var.create(
                [dragged_column.order, dragged_column.order.to(int) + 1]
            ).contains(
                rx.cond(
                    replace_position < 0,
                    n_columns,
                    replace_position,
                )
            )

        return _can_drop


@rx.memo
def item_card(item: ItemInfo, dragging: bool = False):
    return rx.card(
        rx.text(item.title),
        rx.text(item.order, size="1"),
        width="100%",
        background_color=rx.cond(dragging, "green", "inherit"),
    )


@rx.memo
def item_card_drop_target(item: ItemInfo):
    return item_drop_target(
        column_id=item.column_id,
        replace_position=item.order,
        child_after=rxe.dnd.draggable(
            item_card(item=item),
            item={"id": item.id},
            type=ItemInfo.__drag_type__,
            width="100%",
            cursor=rx.cond(
                rxe.dnd.Draggable.collected_params.is_dragging,
                "grabbing",
                "grab",
            ),
            on_end=lambda item: rx.toast(f"You dropped {item.id}"),
        ),
    )


@rx.memo
def item_drop_target(
    column_id: str,
    replace_position: int,
    child_before: rx.Component = rx.fragment(),
    child_after: rx.Component = rx.fragment(),
):
    params = rxe.dnd.DropTarget.collected_params
    dragged_item = KanbanState.item_by_id(params.item.id)
    return rxe.dnd.drop_target(
        rx.vstack(
            child_before,
            rx.cond(
                params.is_over & params.can_drop,
                item_card(item=dragged_item, dragging=True),
            ),
            child_after,
            padding="var(--space-2)",
            width="100%",
        ),
        accept=[ItemInfo.__drag_type__],
        can_drop=KanbanState.can_drop_item(
            column_id=column_id,
            replace_position=replace_position,
        ),
        on_drop=KanbanState.on_item_drop(column_id, replace_position),
        width="100%",
    )


@rx.memo
def column_drop_target(
    replace_position: int,
    child_before: rx.Component = rx.fragment(),
    child_after: rx.Component = rx.fragment(),
):
    params = rxe.dnd.DropTarget.collected_params
    dragged_column = KanbanState.columns.get(params.item.id.to(str), None)
    return rxe.dnd.drop_target(
        rx.hstack(
            child_before,
            rx.cond(
                params.is_over & params.can_drop,
                kanban_column(
                    col=dragged_column,
                    dragging=True,
                ),
                rx.cond(params.is_over, kanban_column(col=dragged_column)),
            ),
            child_after,
            padding="var(--space-1)",
            width="100%",
        ),
        accept=[KanbanColumn.__drag_type__],
        can_drop=KanbanState.can_drop_column(
            replace_position=replace_position,
        ),
        on_drop=KanbanState.on_column_drop(replace_position),
        width="100%",
        height=COL_HEIGHT,
    )


@rx.memo
def kanban_column(
    col: KanbanColumn,
    dragging: bool = False,
):
    """Render a kanban column."""
    return rx.card(
        rx.vstack(
            item_drop_target(
                column_id=col.id,
                replace_position=0,
                child_before=rx.heading(col.title, width="100%"),
            ),
            rx.foreach(
                KanbanState.ordered_items_by_column_id.get(col.id, []),
                lambda item: item_card_drop_target(
                    item=item,
                    key=item.id,
                ),
            ),
            item_drop_target(
                column_id=col.id,
                replace_position=-1,
                child_after=rx.form(
                    rx.hstack(
                        rx.input(placeholder="New Item", name="name", flex_grow="1"),
                        rx.el.input(type="hidden", name="column_id", value=col.id),
                        rx.button("Add"),
                        width="100%",
                    ),
                    on_submit=KanbanState.new_item,
                    reset_on_submit=True,
                    width="100%",
                    cursor="default",
                ),
            ),
            width="100%",
            spacing="0",
        ),
        min_width=COL_MIN_WIDTH,
        background_color=rx.cond(dragging, "green", "inherit"),
        margin_x="0",
    )


@rx.memo
def kanban_column_drop_target(col: KanbanColumn):
    return column_drop_target(
        replace_position=col.order,
        child_after=rxe.dnd.draggable(
            rx.el.div(
                kanban_column(col=col),
                width="100%",
                style={
                    "&": rx.cond(
                        rxe.dnd.Draggable.collected_params.is_dragging,
                        {"display": "none", "cursor": "grabbing"},
                        {"cursor": "move"},
                    )
                },
            ),
            item={"id": col.id},
            type=KanbanColumn.__drag_type__,
        ),
    )


@demo(
    route="/kanban",
    title="Kanban Board Demo",
    description="A multi-draggable UI for task management.",
    on_load=KanbanState.on_load,
)
def kanban_page() -> rx.Component:
    # Welcome Page (Index)
    return rx.container(
        rx.hstack(
            rx.foreach(
                KanbanState.ordered_columns,
                lambda col: kanban_column_drop_target(
                    col=col,
                    key=col.id,
                ),
            ),
            column_drop_target(
                replace_position=-1,
                child_after=rx.form(
                    rx.hstack(
                        rx.input(placeholder="New Column", name="name"),
                        rx.button("Add"),
                        min_width=COL_MIN_WIDTH,
                    ),
                    on_submit=KanbanState.new_column,
                    reset_on_submit=True,
                ),
            ),
            spacing="0",
        ),
    )



"""Popup component for displaying information on a map."""

from .base import BaseLeafletComponent


class Popup(BaseLeafletComponent):
    """Popup component for displaying information on a map."""

    tag = "Popup"
    ...



"""TagsInput component for Mantine."""

from typing import Callable, Literal

import reflex as rx
from reflex.components.component import Component
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import BaseMantineInput


class TagsInput(BaseMantineInput):
    """Display tags for multi-value input."""

    tag = "TagsInput"

    # Props for TagsInput specific functionality
    # Accept values on blur
    accept_value_on_blur: Var[bool]

    # Allow duplicate tags
    allow_duplicates: Var[bool]

    # Props for the clear button
    clear_button_props: Var[dict]

    # Combobox props
    combobox_props: Var[dict]

    # Data displayed in the dropdown. Values must be unique, otherwise an error will be thrown and component will not render.
    data: Var[list[str]]

    # Uncontrolled dropdown initial opened state
    default_dropdown_opened: Var[bool]

    # Default search value
    default_search_value: Var[str]

    # Default value for uncontrolled component
    default_value: Var[list[str]]

    # Description displayed below the input. If not set, the description will not be rendered.
    description: Var[Component | str]

    # Props for the description
    description_props: Var[dict]

    # Sets disabled attribute on the input element
    disabled: Var[bool]

    # Controlled dropdown opened state
    dropdown_opened: Var[bool]

    # Contents of Input.Error component. If not set, error is not rendered.
    error: Var[Component | str | bool]

    # Props for the error message.
    error_props: Var[dict]

    # Function based on which items are filtered and sorted
    filter: Var[Callable]

    # Props passed down to the hidden input
    hidden_input_props: Var[dict]

    # Divider used to separate values in the hidden input value attribute, ',' by default
    hidden_input_values_divider: Var[str]

    # Input container component
    input_container: Var[Component]

    # size prop added to the input element
    input_size: Var[str]

    # Controls order of the elements, ['label', 'description', 'input', 'error'] by default
    input_wrapper_order: Var[list[Literal["label", "description", "input", "error"]]]

    # Label component
    label: Var[Component | str]

    # Label props
    label_props: Var[dict]

    # Maximum number of options displayed at a time, Infinity by default
    limit: Var[int]

    # max-height of the dropdown, only applicable when withScrollArea prop is true, 250 by default
    max_dropdown_height: Var[float | str]

    # Maximum number of tags, Infinity by default
    max_tags: Var[int]

    # Determines whether the input should have cursor: pointer style, false by default
    pointer: Var[bool]

    # Key of theme.radius or any valid CSS value to set border-radius, numbers are converted to rem, theme.defaultRadius by default
    radius: Var[Literal["xs", "sm", "md", "lg", "xl"] | int | str]

    # A function to render content of the option, replaces the default content of the option
    render_option: Var[Callable]

    # Adds required attribute to the input and a red asterisk on the right side of label, false by default
    required: Var[bool]

    # Props passed down to the underlying ScrollArea component in the dropdown
    scroll_area_props: Var[dict]

    # Controlled search value
    search_value: Var[str]

    # Determines whether the first option should be selected when value changes, false by default
    select_first_option_on_change: Var[bool]

    # Controls input height and horizontal padding, 'sm' by default
    size: Var[Literal["xs", "sm", "md", "lg", "xl"]]

    # Characters that should trigger tags split, [','] by default
    split_chars: Var[list[str]]

    # Controlled component value
    value: Var[list[str]]

    # Determines whether the required asterisk should be displayed. Overrides required prop. Does not add required attribute to the input. false by default
    with_asterisk: Var[bool]

    # Determines whether the input should have red border and red text color when the error prop is set, true by default
    with_error_styles: Var[bool]

    # Determines whether the options should be wrapped with ScrollArea.AutoSize, true by default
    with_scroll_area: Var[bool]

    # Event Handlers
    # Called when value changes
    on_change: rx.EventHandler[rx.event.passthrough_event_spec(list[str])]

    # Called when the clear button is clicked
    on_clear: rx.EventHandler[lambda: []]

    # Called when the dropdown is opened
    on_dropdown_open: rx.EventHandler[lambda: []]

    # Called when the dropdown is closed
    on_dropdown_close: rx.EventHandler[lambda: []]

    # Called when user tries to submit a duplicated tag
    on_duplicate: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when option is submitted from dropdown with mouse click or Enter key
    on_option_submit: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when tag is removed
    on_remove: rx.EventHandler[rx.event.passthrough_event_spec(str)]

    # Called when search changes
    on_search_change: rx.EventHandler[lambda value: [value]]


tags_input = TagsInput.create



"""Enterprise environment variables."""

from reflex.environment import EnvironmentVariables, EnvVar, env_var


class EnvironmentEnterpriseVariables(EnvironmentVariables):
    """Enterprise environment variables."""

    # Set the access token needed to authenticate with the Reflex backend.
    REFLEX_ACCESS_TOKEN: EnvVar[str | None] = env_var(None)

    CI: EnvVar[bool] = env_var(False)


environment = EnvironmentEnterpriseVariables()



"""This module provides a drag-and-drop component for Reflex applications."""

import reflex

from . import hooks as api
from . import util
from .flow import (
    Background,
    BaseEdge,
    ControlButton,
    Controls,
    EdgeLabelRenderer,
    EdgeText,
    Flow,
    FlowProvider,
    Handle,
    MiniMap,
    NodeResizeControl,
    NodeResizer,
    NodeToolbar,
    Panel,
    ViewportPortal,
)


class FlowNamespace(reflex.ComponentNamespace):
    """Namespace for all flow-related components."""

    __call__ = Flow.create
    provider = FlowProvider.create
    background = Background.create
    base_edge = BaseEdge.create
    control_button = ControlButton.create
    controls = Controls.create
    edge_label_renderer = EdgeLabelRenderer.create
    edge_text = EdgeText.create
    handle = Handle.create
    mini_map = MiniMap.create
    node_resize_control = NodeResizeControl.create
    node_resizer = NodeResizer.create
    node_toolbar = NodeToolbar.create
    panel = Panel.create
    viewport_portal = ViewportPortal.create
    util = util
    api = api


flow = FlowNamespace()

__all__ = ["flow"]



"""Map module for Reflex Enterprise."""

from __future__ import annotations

from reflex.components.component import ComponentNamespace

from .base import MapAPI
from .controls import (
    AttributionControl,
    LayersControl,
    LayersControlBaseLayer,  # noqa: F401
    LayersControlOverlay,  # noqa: F401
    ScaleControl,
    ZoomControl,
)
from .map_container import MapContainer
from .marker import Marker
from .popup import Popup
from .tile_layer import TileLayer
from .tooltip import Tooltip
from .types import (
    LatLng,
    LatLngBounds,
    LocateOptions,
    ZoomEvent,
    ZoomLevelsChangeEvent,
    latlng,
    latlng_bounds,
    locate_options,
)
from .vectors import (
    Circle,
    CircleMarker,
    PathOptions,
    Polygon,
    Polyline,
    Rectangle,
    SVGOverlay,
    path_options,
)


class MapNamespace(ComponentNamespace):
    """Namespace for map components."""

    __call__ = staticmethod(MapContainer.create)

    # Controls
    zoom_control = staticmethod(ZoomControl.create)
    attribution_control = staticmethod(AttributionControl.create)
    scale_control = staticmethod(ScaleControl.create)
    layers_control = staticmethod(LayersControl.create)
    # layers_control_base_layer = staticmethod(LayersControlBaseLayer.create)
    # layers_control_overlay = staticmethod(LayersControlOverlay.create)

    # Basic layers
    tile_layer = staticmethod(TileLayer.create)
    marker = staticmethod(Marker.create)
    popup = staticmethod(Popup.create)
    tooltip = staticmethod(Tooltip.create)

    # Vector layers
    circle = staticmethod(Circle.create)
    circle_marker = staticmethod(CircleMarker.create)
    polyline = staticmethod(Polyline.create)
    polygon = staticmethod(Polygon.create)
    rectangle = staticmethod(Rectangle.create)
    svg_overlay = staticmethod(SVGOverlay.create)

    # Events and types
    ZoomEvent: type[ZoomEvent] = ZoomEvent
    ZoomLevelsChangeEvent: type[ZoomLevelsChangeEvent] = ZoomLevelsChangeEvent
    LatLng: type[LatLng] = LatLng
    LatLngBounds: type[LatLngBounds] = LatLngBounds
    LocateOptions: type[LocateOptions] = LocateOptions
    PathOptions: type[PathOptions] = PathOptions

    # Helper functions
    latlng = staticmethod(latlng)
    latlng_bounds = staticmethod(latlng_bounds)
    locate_options = staticmethod(locate_options)
    path_options = staticmethod(path_options)
    api = staticmethod(MapAPI.create)


map = MapNamespace()

__all__ = ["map"]



"""Stub file for reflex_enterprise/components/mantine/autocomplete.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from typing import Any, Literal, Mapping, Optional, Sequence

from reflex.components.component import Component
from reflex.components.core.breakpoints import Breakpoints
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var

from reflex_enterprise.components.mantine.base import (
    MantineDescriptionProps,
    MantineErrorProps,
    MantineLabelProps,
)
from reflex_enterprise.components.mantine.combobox import Combobox, MiddlewaresProps

class Autocomplete(
    MantineDescriptionProps, MantineErrorProps, MantineLabelProps, Combobox
):
    @classmethod
    def create(
        cls,
        *children,
        clear_button_props: Var[dict] | dict | None = None,
        clearable: Var[bool] | bool | None = None,
        combobox_props: Var[dict] | dict | None = None,
        data: Var[list[str]] | list[str] | None = None,
        default_dropdown_opened: Var[bool] | bool | None = None,
        default_value: Var[list[str]] | list[str] | None = None,
        disabled: Var[bool] | bool | None = None,
        dropdown_opened: Var[bool] | bool | None = None,
        dropdown_max_height: Var[float | str] | float | str | None = None,
        size: Literal["lg", "md", "sm", "xl", "xs"]
        | Var[Literal["lg", "md", "sm", "xl", "xs"] | str]
        | str
        | None = None,
        value: Var[str] | str | None = None,
        description: Component | Var[Component | str] | str | None = None,
        description_props: Var[dict] | dict | None = None,
        error: Component | Var[Component | bool | str] | bool | str | None = None,
        error_props: Var[dict] | dict | None = None,
        label: Component | Var[Component | str] | str | None = None,
        label_props: Var[dict] | dict | None = None,
        arrow_offset: Var[int] | int | None = None,
        arrow_position: Literal["center", "side"]
        | Var[Literal["center", "side"]]
        | None = None,
        arrow_radius: Var[int] | int | None = None,
        arrow_size: Var[int] | int | None = None,
        dropdown_padding: Var[int | str] | int | str | None = None,
        floating_strategy: Literal["absolute", "fixed"]
        | Var[Literal["absolute", "fixed"]]
        | None = None,
        hide_detached: Var[bool] | bool | None = None,
        keep_mounted: Var[bool] | bool | None = None,
        middlewares: MiddlewaresProps
        | Var[MiddlewaresProps | dict[str, Any]]
        | dict[str, Any]
        | None = None,
        offset: Var[int] | int | None = None,
        overlay_props: Var[dict[str, Any]] | dict[str, Any] | None = None,
        portal_props: Var[dict[str, Any]] | dict[str, Any] | None = None,
        position: Literal["bottom", "left", "right", "top"]
        | Var[Literal["bottom", "left", "right", "top"]]
        | None = None,
        position_dependencies: Var[list[Any]] | list[Any] | None = None,
        radius: Var[int | str] | int | str | None = None,
        reset_selection_on_option_hover: Var[bool] | bool | None = None,
        return_focus: Var[bool] | bool | None = None,
        shadow: Var[str] | str | None = None,
        store: Any | Var[Any] | None = None,
        transition_props: Var[dict[str, Any]] | dict[str, Any] | None = None,
        width: Literal["target"]
        | Var[Literal["target"] | int | str]
        | int
        | str
        | None = None,
        with_arrow: Var[bool] | bool | None = None,
        with_overlay: Var[bool] | bool | None = None,
        with_portal: Var[bool] | bool | None = None,
        z_index: Var[int] | int | None = None,
        placeholder: Var[str] | str | None = None,
        multiline: Var[bool] | bool | None = None,
        read_only: Var[bool] | bool | None = None,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        left_section: Component | Var[Component | str] | str | None = None,
        left_section_props: Var[dict] | dict | None = None,
        left_section_width: Var[float | str] | float | str | None = None,
        left_section_pointer_events: Var[str] | str | None = None,
        right_section: Component | Var[Component | str] | str | None = None,
        right_section_props: Var[dict] | dict | None = None,
        right_section_width: Var[float | str] | float | str | None = None,
        right_section_pointer_events: Var[str] | str | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_close: Optional[EventType[()]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_dismiss: Optional[EventType[()]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_dropdown_close: Optional[EventType[()]] = None,
        on_dropdown_open: Optional[EventType[()]] = None,
        on_enter_transition_end: Optional[EventType[()]] = None,
        on_exit_transition_end: Optional[EventType[()]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_open: Optional[EventType[()]] = None,
        on_option_submit: Optional[EventType[Any]] = None,
        on_position_change: Optional[EventType[()] | EventType[float]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "Autocomplete":
        """Create the Reflex Enterprise component."""
        ...

autocomplete = Autocomplete.create



"""Stub file for reflex_enterprise/components/map/base.py"""

# ------------------- DO NOT EDIT ----------------------
# This file was generated by `reflex/utils/pyi_generator.py`!
# ------------------------------------------------------
from dataclasses import dataclass
from typing import Any, Mapping, Optional, Sequence

import reflex as rx
from reflex import ImportVar, constants
from reflex.components.core.breakpoints import Breakpoints
from reflex.event import EventType, PointerEventInfo
from reflex.vars.base import Var, VarData
from reflex.vars.object import ObjectVar

from reflex_enterprise.components.component import NoSSRComponentEnterprise
from reflex_enterprise.vars import JSAPIVar

PACKAGE_NAME = "react-leaflet"
PACKAGE_VERSION = "5.0.0"
path = rx.asset("MapLazyComponents.jsx", shared=True)
public_path = "$/public" + path
refs_var_data = VarData(
    imports={f"$/{constants.Dirs.STATE_PATH}": [ImportVar(tag="refs")]}
)
refs = Var(_js_expr="refs", _var_data=refs_var_data).to(ObjectVar, Mapping[str, str])

class MapAPIVar(JSAPIVar): ...

@dataclass
class MapAPI:
    ref: str

    @classmethod
    def create(cls, ref: str) -> MapAPI: ...

class BaseLeafletComponent(NoSSRComponentEnterprise):
    def add_imports(self): ...
    @classmethod
    def create(
        cls,
        *children,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "BaseLeafletComponent":
        """Create the Reflex Enterprise component."""
        ...

class LazyBaseLeafletComponent(BaseLeafletComponent):
    @classmethod
    def create(
        cls,
        *children,
        style: Sequence[Mapping[str, Any]]
        | Mapping[str, Any]
        | Var[Mapping[str, Any]]
        | Breakpoints
        | None = None,
        key: Any | None = None,
        id: Any | None = None,
        ref: Var | None = None,
        class_name: Any | None = None,
        custom_attrs: dict[str, Var | Any] | None = None,
        on_blur: Optional[EventType[()]] = None,
        on_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_context_menu: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_double_click: Optional[EventType[()] | EventType[PointerEventInfo]] = None,
        on_focus: Optional[EventType[()]] = None,
        on_mount: Optional[EventType[()]] = None,
        on_mouse_down: Optional[EventType[()]] = None,
        on_mouse_enter: Optional[EventType[()]] = None,
        on_mouse_leave: Optional[EventType[()]] = None,
        on_mouse_move: Optional[EventType[()]] = None,
        on_mouse_out: Optional[EventType[()]] = None,
        on_mouse_over: Optional[EventType[()]] = None,
        on_mouse_up: Optional[EventType[()]] = None,
        on_scroll: Optional[EventType[()]] = None,
        on_scroll_end: Optional[EventType[()]] = None,
        on_unmount: Optional[EventType[()]] = None,
        **props,
    ) -> "LazyBaseLeafletComponent":
        """Create the Reflex Enterprise component."""
        ...



"""Map container component for leaflet."""

from __future__ import annotations

from reflex.vars.base import Var

from reflex_enterprise.components.map.base import LazyBaseLeafletComponent
from reflex_enterprise.components.map.map_consumer import MapConsumer
from reflex_enterprise.components.map.types import LatLng, LatLngBounds


class MapContainerControlOptions(LazyBaseLeafletComponent):
    """MapContainerControlOptions component for leaflet."""

    # Whether a attribution control is added to the map by default.
    attribution_control: Var[bool]

    # Whether a zoom control is added to the map by default.
    zoom_control: Var[bool]


class MapContainerInteractionOptions(LazyBaseLeafletComponent):
    """MapContainerInteractionOptions component for leaflet."""

    # Whether the map can be zoomed to a rectangular area specified by dragging the mouse while pressing the shift key.
    box_zoom: Var[bool]

    # Whether the map can be zoomed in by double clicking on it and zoomed out by double clicking while holding shift. If passed 'center', double-click zoom will zoom to the center of the view regardless of where the mouse was.
    double_click_zoom: Var[bool | str]

    # Whether the map is draggable with mouse/touch or not.
    dragging: Var[bool]

    # Forces the map's zoom level to always be a multiple of this, particularly right after a fitBounds() or a pinch-zoom. By default, the zoom level snaps to the nearest integer; lower values (e.g. 0.5 or 0.1) allow for greater granularity. A value of 0 means the zoom level will not be snapped after fitBounds or a pinch-zoom.
    zoom_snap: Var[float]

    # Controls how much the map's zoom level will change after a zoomIn(), zoomOut(), pressing + or - on the keyboard, or using the zoom controls. Values smaller than 1 (e.g. 0.5) allow for greater granularity.
    zoom_delta: Var[float]

    # Whether the map automatically handles browser window resize to update itself.
    track_resize: Var[bool]


class MapContainerPanningInertiaOptions(LazyBaseLeafletComponent):
    """MapContainerPanningInertiaOptions component for leaflet."""

    # If enabled, panning of the map will have an inertia effect where the map builds momentum while dragging and continues moving in the same direction for some time. Feels especially nice on touch devices. Enabled by default.
    inertia: Var[bool]

    # The rate with which the inertial movement slows down, in pixels/second².
    inertia_deceleration: Var[int]

    # Max speed of the inertial movement, in pixels/second.
    inertia_max_speed: Var[int]

    #
    ease_linearity: Var[float]

    # Whether the map pans when the mouse is dragged outside its bounds.
    world_copy_jump: Var[bool]

    # If maxBounds is set, this option will control how solid the bounds are when dragging the map around. The default value of 0.0 allows the user to drag outside the bounds at normal speed, higher values will slow down map dragging outside bounds, and 1.0 makes the bounds fully solid, preventing the user from dragging outside the bounds.
    max_bounds_viscosity: Var[float]


class MapContainerKeyboardNavigationOptions(LazyBaseLeafletComponent):
    """MapContainerKeyboardNavigationOptions component for leaflet."""

    # Makes the map focusable and allows users to navigate the map with keyboard arrows and +/- keys.
    keyboard: Var[bool]

    # Amount of pixels to pan when pressing an arrow key.
    keyboard_pan_delta: Var[int]


class MapContainerMouseWheelOptions(LazyBaseLeafletComponent):
    """MapContainerMouseWheelOptions component for leaflet."""

    # Whether the map can be zoomed by using the mouse wheel. If passed 'center', it will zoom to the center of the view regardless of where the mouse was.
    scroll_wheel_zoom: Var[bool | str]

    # Limits the rate at which a wheel can fire (in milliseconds). By default user can't zoom via wheel more often than once per 40 ms.
    wheel_debounce_time: Var[int]

    # How many scroll pixels (as reported by L.DomEvent.getWheelDelta) mean a change of one full zoom level. Smaller values will make wheel-zooming faster (and vice versa).
    wheel_px_per_zoom_level: Var[int]

    # Whether the map can be scrolled with a mouse wheel.
    wheel_delta: Var[int]


class MapContainerTouchInteractionOptions(LazyBaseLeafletComponent):
    """MapContainerTouchInteractionOptions component for leaflet."""

    # Enables simulation of contextmenu event, default is true for mobile Safari.
    tap_hold: Var[bool]

    # The max number of pixels a user can shift his finger during touch for it to be considered a valid tap.
    tap_tolerance: Var[int]

    # Whether the map can be zoomed by touch-dragging with two fingers. If passed 'center', it will zoom to the center of the view regardless of where the touch events (fingers) were. Enabled for touch-capable web browsers.
    touch_zoom: Var[bool | str]

    # Set it to false if you don't want the map to zoom beyond min/max zoom and then bounce back when pinch-zooming.
    bounce_at_zoom_limits: Var[bool]


class MapContainerMapStateOptions(LazyBaseLeafletComponent):
    """MapContainerMapStateOtions component for leaflet."""

    # The Coordinate Reference System to use. Don't change this if you're not sure what it means.
    crs: Var[str]

    # Initial geographic center of the map view.
    center: Var[LatLng]

    # Initial zoom level of the map view.
    zoom: Var[float]

    # Minimum zoom level of the map. If not specified and at least one GridLayer or TileLayer is in the map, the lowest of their minZoom options will be used instead.
    min_zoom: Var[int]

    # Maximum zoom level of the map. If not specified and at least one GridLayer or TileLayer is in the map, the highest of their maxZoom options will be used instead.
    max_zoom: Var[int]

    # The bounds of the map view. If not specified, the map will be initialized to show the whole world.
    max_bounds: Var[LatLngBounds]


class MapContainerAnimationOptions(LazyBaseLeafletComponent):
    """MapContainerAnimationOptions component for leaflet."""

    # Whether the map zoom animation is enabled. By default it's enabled in all browsers that support CSS3 Transitions except Android.
    zoom_animation: Var[bool]

    # Won't animate zoom if the zoom difference exceeds this value.
    zoom_animation_threshold: Var[int]

    # Whether the tile fade animation is enabled. By default it's enabled in all browsers that support CSS3 Transitions except Android.
    fade_animation: Var[int]

    # Whether markers animate their zoom with the zoom animation, if disabled they will disappear for the length of the animation. By default it's enabled in all browsers that support CSS3 Transitions except Android.
    marker_zoom_animation: Var[bool]

    # Defines the maximum size of a CSS translation transform. The default value should not be changed unless a web browser positions layers in the wrong place after doing a large panBy.
    transform_3d_limit: Var[int]


class MapContainer(
    MapContainerControlOptions,
    MapContainerInteractionOptions,
    MapContainerPanningInertiaOptions,
    MapContainerKeyboardNavigationOptions,
    MapContainerMouseWheelOptions,
    MapContainerTouchInteractionOptions,
    MapContainerMapStateOptions,
    MapContainerAnimationOptions,
    LazyBaseLeafletComponent,
):
    """MapContainer component for leaflet."""

    tag = "MapContainer"

    # Whether Paths should be rendered on a Canvas renderer. By default, all Paths are rendered in a SVG renderer.
    prefer_canvas: Var[bool]

    @classmethod
    def create(cls, *children, id: str, **props):
        """Create a MapContainer."""
        # Extract event props for the MapConsumer, but exclude on_mount and on_unmount
        events_props = {
            key: value
            for key, value in props.items()
            if key.startswith("on_") and key not in ("on_mount", "on_unmount")
        }

        # Remove extracted event props from the main props
        for key in events_props:
            if key in props:
                props.pop(key)

        # Add the MapConsumer if we have events
        children = (*children, MapConsumer.create(map_ref=id, **events_props))

        return super().create(*children, **props)

    def add_custom_code(self) -> list[str]:
        """Add custom code for leaflet components."""
        return []



"""Column definition for ag-grid."""

from typing import Any, Callable, Literal

from reflex.components.props import PropsBase
from reflex.event import EventChain
from reflex.vars.base import Var
from reflex.vars.function import ArgsFunctionOperation

from ..resource import AGEditors, AGFilters
from .base import JS_EXPRESSION, AgGridResourceBase


class ColumnBaseProps(PropsBase):
    """Base column properties for ag-grid."""

    # The field of the row object to get the cell's data from. Deep references into a row object is supported via dot notation, i.e 'address.firstLine'.
    field: str | Var[str] | None = None

    # The unique ID to give the column. This is optional. If missing, the ID will default to the field. If both field and colId are missing, a unique ID will be generated. This ID is used to identify the column in the API for sorting, filtering etc.
    col_id: str | Var[str] | None = None

    # A comma separated string or array of strings containing ColumnType keys which can be used as a template for a column. This helps to reduce duplication of properties when you have a lot of common column properties.
    type: str | Var[str] | None = None

    # The data type of the cell values for this column. Can either infer the data type from the row data (true - the default behaviour), define a specific data type (string), or have no data type (false). If setting a specific data type (string value), this can either be one of the pre-defined data types 'text', 'number', 'boolean', 'date', 'dateString' or 'object', or a custom data type that has been defined in the dataTypeDefinitions grid option. Data type inference only works for the Client-Side Row Model, and requires non-null data. It will also not work if the valueGetter, valueParser or refData properties are defined, or if this column is a sparkline.
    cell_data_type: bool | str | Var[bool] | Var[str] | None = None

    # Function or expression. Gets the value from your data for display.
    value_getter: JS_EXPRESSION | None = None

    # A function or expression to format a value, should return a string.
    value_formatter: JS_EXPRESSION | None = None

    # Provided a reference data map to be used to map column values to their respective value from the map.
    ref_data: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Function to return a string key for a value. This string is used for grouping, Set filtering, and searching within cell editor dropdowns. When filtering and searching the string is exposed to the user, so make sure to return a human-readable value.
    key_creator: JS_EXPRESSION | None = None

    # Custom comparator for values, used by renderer to know if values have changed. Cells whose values have not changed don't get refreshed. By default the grid uses === which should work for most use cases.
    equals: JS_EXPRESSION | None = None

    # CSS class to use for the tool panel cell. Can be a string, array of strings, or function.
    tool_panel_class: str | list[str] | Var[str] | Var[list[str]] | None = None

    # Set to true if you do not want this column or group to appear in the Columns Tool Panel.
    suppress_columns_tool_panel: bool | Var[bool] | None = None

    # Whether to only show the column when the group is open / closed. If not set the column is always displayed as part of the group.
    column_group_show: Literal["open", "closed"] | Var[str] | None = None

    # Icons to use inside the column instead of the grid's default icons. Leave undefined to use defaults.
    icons: dict[str, str] | Var[dict[str, str]] | None = None

    # Set to true if this column is not navigable (i.e. cannot be tabbed into), otherwise false. Can also be a callback function to have different rows navigable.
    suppress_navigable: bool | Var[bool] | None = None

    # Allows the user to suppress certain keyboard events in the grid cell.
    suppress_keyboard_event: Var[Callable] | None = None

    # Pasting is on by default as long as cells are editable (non-editable cells cannot be modified, even with a paste operation). Set to true turn paste operations off.
    suppress_paste: bool | Var[bool] | None = None

    # Set to true to prevent the fillHandle from being rendered in any cell that belongs to this column
    suppress_fill_handle: bool | Var[bool] | None = None

    # Customise the list of menu items available in the context menu.
    context_menu_items: list[str] | Var[list[str]] | None = None

    # Context property that can be used to associate arbitrary application data with this column definition.
    context: Any | None = None


class ColumnAccessibilityProps(PropsBase):
    """Column accessibility properties for ag-grid."""

    # Used for screen reader announcements - the role property of the cells that belong to this column.
    cell_aria_role: str | Var[str] | None = None


class ColumnAggregationProps(PropsBase):
    """Column aggregation properties for ag-grid."""

    # Name of function to use for aggregation. In-built options are: sum, min, max, count, avg, first, last. Also accepts a custom aggregation name or an aggregation function.
    agg_func: str | Var[str] | None = None

    # Same as aggFunc, except only applied when creating a new column. Not applied when updating column definitions.
    initial_agg_func: str | Var[str] | None = None

    # Set to true if you want to be able to aggregate by this column via the GUI. This will not block the API or properties being used to achieve aggregation.
    enable_value: bool | Var[bool] | None = None

    # Aggregation functions allowed on this column e.g. ['sum', 'avg']. If missing, all installed functions are allowed. This will only restrict what the GUI allows a user to select, it does not impact when you set a function via the API.
    allowed_agg_funcs: list[str] | Var[list[str]] | None = None

    # The name of the aggregation function to use for this column when it is enabled via the GUI. Note that this does not immediately apply the aggregation function like aggFunc
    default_agg_func: str | Var[str] | None = None


class ColumnDisplayProps(PropsBase):
    """Column display properties for ag-grid."""

    # Set to true for this column to be hidden.
    hide: bool | Var[bool] | None = None

    # Same as hide, except only applied when creating a new column. Not applied when updating column definitions.
    initial_hide: bool | Var[bool] | None = None

    # Set to true to block making column visible / hidden via the UI (API will still work).
    lock_visible: bool | Var[bool] | None = None

    # Lock a column to position to 'left' or'right' to always have this column displayed in that position. true is treated as 'left'
    lock_position: Literal["left", "right"] | bool | None = None

    # Set to true if you do not want this column to be movable via dragging.
    suppress_movable: bool | Var[bool] | None = None

    # By default, values are formatted using the column's valueFormatter when exporting data from the grid. This applies to CSV and Excel export, as well as clipboard operations and the fill handle. Set to false to prevent values from being formatted for these operations. Regardless of this option, if custom handling is provided for the export operation, the value formatter will not be used.
    use_value_formatter_for_export: bool | Var[bool] | None = None


class ColumnEditingProps(PropsBase):
    """Column editing properties for ag-grid."""

    # Set to true if this column is editable, otherwise false. Can also be a function to have different rows editable.
    editable: bool | Var[bool] | JS_EXPRESSION | None = None

    # Function or expression. Custom function to modify your data based off the new value for saving. Return true if the data changed.
    value_setter: EventChain | Var[EventChain] | None = None

    # Function or expression. Parses the value for saving.
    value_parser: Var[Callable] | None = None

    # Provide your own cell editor component for this column's cells.
    cell_editor: AGEditors | str | Var[AGEditors] | Var[str] | None = None

    # Params to be passed to the cellEditor component.
    cell_editor_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Callback to select which cell editor to be used for a given row within the same column.
    cell_editor_selector: Var[Callable] | None = None

    # Set to true, to have the cell editor appear in a popup.
    cell_editor_popup: bool | None = None

    # Set the position for the popup cell editor. Possible values are
    #   over: Popup will be positioned over the cell
    #   under: Popup will be positioned below the cell leaving the cell value visible.
    cell_editor_popup_position: Literal["over", "under"] | None = None

    # Set to true to have cells under this column enter edit mode after single click.
    single_click_edit: bool | None = None

    # By default, values are parsed using the column's valueParser when importing data to the grid. This applies to clipboard operations and the fill handle. Set to false to prevent values from being parsed for these operations. Regardless of this option, if custom handling is provided for the import operation, the value parser will not be used.
    use_value_parser_for_import: bool | Var[bool] | None = None


class ColumnEventProps(PropsBase):
    """Column event properties for ag-grid."""

    # Callback for after the value of a cell has changed, either due to editing or the application calling api.setValue().
    on_cell_value_changed: EventChain | Var[EventChain] | None = None

    # Callback called when a cell is clicked.
    on_cell_clicked: EventChain | Var[EventChain] | None = None

    # Callback called when a cell is double clicked.
    on_cell_double_clicked: EventChain | Var[EventChain] | None = None

    # Callback called when a cell is right clicked.
    on_cell_context_menu: EventChain | Var[EventChain] | None = None


class ColumnFilterProps(PropsBase):
    """Column filter properties for ag-grid."""

    # Filter component to use for this column. Set to true to use the default filter. Set to the name of a Provided Filter or set to a IFilterComp.
    filter: AGFilters | str | Var[AGFilters] | Var[str] | None = None

    # Params to be passed to the filter component specified in filter.
    filter_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Function or expression. Gets the value for filtering purposes.
    filter_value_getter: JS_EXPRESSION | None = None

    # A function to tell the grid what Quick Filter text to use for this column if you don't want to use the default (which is calling toString on the value).
    get_quick_filter_text: Var[Callable] | None = None

    # Whether to display a floating filter for this column.
    floating_filter: bool | Var[bool] | None = None

    # The custom component to be used for rendering the floating filter. If none is specified the default AG Grid is used.
    floating_filter_component: Any | None = None

    # Params to be passed to floatingFilterComponent.
    floating_filter_component_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Set to true if you do not want this column (filter) or group (filter group) to appear in the Filters Tool Panel.
    suppress_filters_tool_panel: bool | Var[bool] | None = None


class ColumnHeaderProps(PropsBase):
    """Column header properties for ag-grid."""

    # The name to render in the column header.
    header_name: str | Var[str] | None = None

    # Function or expression. Gets the value for display in the header.
    header_value_getter: Var | str | None = None

    # Tooltip for the column header
    header_tooltip: str | Var[str] | None = None

    # An object of CSS values / or function returning an object of CSS values for a particular header.
    header_style: dict[str, Any] | Var[dict[str, Any]] | None = None

    # CSS class to use for the header cell. Can be a string, array of strings, or function.
    header_class: str | list[str] | Var[str] | Var[list[str]] | None = None

    # The custom header group component to be used for rendering the component header. If none specified the default AG Grid is used.
    header_component: Any | None = None

    # The parameters to be passed to the headerComponent.
    header_component_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # If enabled then column header names that are too long for the column width will wrap onto the next line. Default false
    wrap_header_text: bool | None = None

    # If enabled then the column header row will automatically adjust height to accommodate the size of the header cell. This can be useful when using your own headerComponent or long header names in conjunction with wrapHeaderText.
    auto_header_height: bool | None = None

    # Set to an array containing zero, one or many of the following options: 'filterMenuTab' | 'generalMenuTab' | 'columnsMenuTab'. This is used to figure out which menu tabs are present and in which order the tabs are shown.
    menu_tabs: list[str] | Var[list[str]] | None = None

    # Params used to change the behaviour and appearance of the Column Chooser/Columns Menu tab.
    columns_chooser_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Customise the list of menu items available in the column menu.
    main_menu_items: list[str] | Var[list[str]] | None = None

    # Set to true if no menu button should be shown for this column header.
    suppress_header_menu_button: bool | Var[bool] | None = None

    # Set to true to not display the filter button in the column header. Doesn't apply when columnMenu = 'legacy'.
    suppress_header_filter_button: bool | Var[bool] | None = None

    # Set to true to not display the column menu when the column header is right-clicked. Doesn't apply when columnMenu = 'legacy'.
    suppress_header_context_menu: bool | Var[bool] | None = None

    # Suppress the grid taking action for the relevant keyboard event when a header is focused.
    suppress_header_keyboard_event: Var[Callable] | None = None

    # If true, the button in the floating filter that opens the parent filter in a popup will not be displayed. Only applies if floatingFilter = true.
    suppress_floating_filter_button: bool | None = None


class ColumnIntegratedChartProps(PropsBase):
    """Column integrated chart properties for ag-grid."""

    # Defines the chart data type that should be used for a column.
    chart_data_type: str | Var[str] | None = None


class ColumnPinnedProps(PropsBase):
    """Column pinned properties for ag-grid."""

    # Pin a column to one side: right or left. A value of true is converted to 'left'.
    pinned: Literal["left", "right"] | bool | None = None

    # Same as pinned, except only applied when creating a new column. Not applied when updating column definitions.
    initial_pinned: Literal["left", "right"] | bool | None = None

    # Set to true to block the user pinning the column, the column can only be pinned via definitions or API.
    lock_pinned: bool | Var[bool] | None = None


class ColumnPivotProps(PropsBase):
    """Column pivot properties for ag-grid."""

    # Set to true to pivot by this column.
    pivot: bool | Var[bool] | None = None

    # Same as pivot, except only applied when creating a new column. Not applied when updating column definitions.
    initial_pivot: bool | Var[bool] | None = None

    # Set this in columns you want to pivot by. If only pivoting by one column, set this to any number (e.g. 0). If pivoting by multiple columns, set this to where you want this column to be in the order of pivots (e.g. 0 for first, 1 for second, and so on).
    pivot_index: int | Var[int] | None = None

    # Same as pivotIndex, except only applied when creating a new column. Not applied when updating column definitions.
    initial_pivot_index: int | Var[int] | None = None

    # Set to true if you want to be able to pivot by this column via the GUI. This will not block the API or properties being used to achieve pivot.
    enable_pivot: bool | Var[bool] | None = None

    # Only for CSRM, see SSRM Pivoting. Comparator to use when ordering the pivot columns, when this column is used to pivot on. The values will always be strings, as the pivot service uses strings as keys for the pivot groups.
    pivot_comparator: Var[Callable] | None = None


class ColumnRenderingProps(PropsBase):
    """Column rendering properties for ag-grid."""

    # An object of CSS values / or function returning an object of CSS values for a particular cell.
    cell_style: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Class to use for the cell. Can be string, array of strings, or function that returns a string or array of strings.
    cell_class: str | list[str] | Var[str] | Var[list[str]] | None = None

    # Rules which can be applied to include certain CSS classes.
    cell_class_rules: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Provide your own cell Renderer component for this column's cells.
    cell_renderer: ArgsFunctionOperation | str | JS_EXPRESSION | None = None

    # Params to be passed to the cellRenderer component.
    cell_renderer_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Callback to select which cell renderer to be used for a given row within the same column.
    cell_renderer_selector: Var[Callable] | None = None

    # Provide your own cell loading Renderer component for this column's cells when using SSRM.
    loading_cell_renderer: ArgsFunctionOperation | str | JS_EXPRESSION | None = None

    # Params to be passed to the loadingCellRenderer component.
    loading_cell_renderer_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Callback to select which loading renderer to be used for a given row within the same column.
    loading_cell_renderer_selector: Var[Callable] | None = None

    # Set to true to have the grid calculate the height of a row based on contents of this column.
    auto_height: bool | None = None

    # Set to true to have the text wrap inside the cell - typically used with autoHeight.
    wrap_text: bool | None = None

    # Set to true to flash a cell when it's refreshed.
    enable_cell_change_flash: bool | None = None


class ColumnRowDragProps(PropsBase):
    """Column row drag properties for ag-grid."""

    # boolean or Function. Set to true (or return true from function) to allow row dragging.
    row_drag: bool | Var[bool] | Var[Callable] | None = None

    # A callback that should return a string to be displayed by the rowDragComp while dragging a row. If this callback is not set, the rowDragText callback in the gridOptions will be used and if there is no callback in the gridOptions the current cell value will be used.
    row_drag_text: Var[Callable] | None = None

    # boolean or Function. Set to true (or return true from function) to allow dragging for native drag and drop.
    dnd_source: bool | Var[bool] | Var[Callable] | None = None

    # Function to allow custom drag functionality for native drag and drop.
    dnd_source_on_row_drag: Var[Callable] | None = None


class ColumnRowGroupProps(PropsBase):
    """Column row group properties for ag-grid."""

    # Set to true to row group by this column.
    row_group: bool | Var[bool] | None = None

    # Same as rowGroup, except only applied when creating a new column. Not applied when updating column definitions.
    initial_row_group: bool | Var[bool] | None = None

    # Set this in columns you want to group by. If only grouping by one column, set this to any number (e.g. 0). If grouping by multiple columns, set this to where you want this column to be in the group (e.g. 0 for first, 1 for second, and so on).
    row_group_index: int | Var[int] | None = None

    # Same as rowGroupIndex, except only applied when creating a new column. Not applied when updating column definitions.
    initial_row_group_index: int | Var[int] | None = None

    # Set to true if you want to be able to row group by this column via the GUI. This will not block the API or properties being used to achieve row grouping.
    enable_row_group: bool | Var[bool] | None = None

    # Set to true to have the grid place the values for the group into the cell, or put the name of a grouped column to just show that group.
    show_row_group: bool | str | None = None


class ColumnSortProps(PropsBase):
    """Column sort properties for ag-grid."""

    # Set to false to disable sorting which is enabled by default.
    sortable: bool | Var[bool] | None = None

    # If sorting by default, set it here. Set to asc or desc.
    sort: Literal["asc", "desc"] | None = None

    # Same as sort, except only applied when creating a new column. Not applied when updating column definitions.
    initial_sort: Literal["asc", "desc"] | None = None

    # If sorting more than one column by default, specifies order in which the sorting should be applied.
    sort_index: int | Var[int] | None = None

    # Same as sortIndex, except only applied when creating a new column. Not applied when updating column definitions.
    initial_sort_index: int | Var[int] | None = None

    # Array defining the order in which sorting occurs (if sorting is enabled). An array with any of the following in any order ['asc','desc',null]
    sorting_order: (
        list[Literal["asc", "desc", None]]
        | Var[list[Literal["asc", "desc", None]]]
        | None
    ) = None

    # Override the default sorting order by providing a custom sort comparator.
    comparator: Var[Callable] | None = None

    # Set to true if you want the unsorted icon to be shown when no sort is applied to this column.
    un_sort_icon: bool | None = None


class ColumnSpanProps(PropsBase):
    """Column span properties for ag-grid."""

    # By default, each cell will take up the width of one column. You can change this behaviour to allow cells to span multiple columns.
    col_span: Var[Callable] | None = None

    # Set to true to automatically merge cells in this column with equal values. Provide a callback to specify custom merging logic.
    span_rows: bool | Var[bool] | None = None


class ColumnTooltipProps(PropsBase):
    """Column tooltip properties for ag-grid."""

    # The field of the tooltip to apply to the cell.
    tooltip_field: str | Var[str] | None = None

    # Callback that should return the string to use for a tooltip, tooltipField takes precedence if set. If using a custom tooltipComponent you may return any custom value to be passed to your tooltip component.
    tooltip_value_getter: Var[Callable] | None = None

    # Provide your own tooltip component for the column.
    tooltip_component: Any | None = None

    # Params to be passed to the tooltipComponent.
    tooltip_component_params: dict[str, Any] | Var[dict[str, Any]] | None = None


class ColumnWidthProps(PropsBase):
    """Column width properties for ag-grid."""

    # The width for this column. If flex is used, this property is ignored.
    width: int | Var[int] | None = None

    # Same as width, except only applied when creating a new column. Not applied when updating column definitions.
    initial_width: int | Var[int] | None = None

    # The minimum width for this column.
    min_width: int | Var[int] | None = None

    # The maximum width for this column.
    max_width: int | Var[int] | None = None

    # The flex value for this column. If flex is used, the width property is ignored.
    flex: int | Var[int] | None = None

    # Same as flex, except only applied when creating a new column. Not applied when updating column definitions.
    initial_flex: int | Var[int] | None = None

    # Set to false to disable resizing which is enabled by default.
    resizable: bool | Var[bool] | None = None

    # Set to true if you want this column's width to be fixed during 'size to fit' operations.
    suppress_size_to_fit: bool | Var[bool] | None = None

    # Set to true if you do not want this column to be auto-resizable by double clicking it's edge.
    suppress_auto_size: bool | Var[bool] | None = None


class ColumnDef(
    ColumnBaseProps,
    ColumnAccessibilityProps,
    ColumnAggregationProps,
    ColumnDisplayProps,
    ColumnEditingProps,
    ColumnEventProps,
    ColumnFilterProps,
    ColumnHeaderProps,
    ColumnIntegratedChartProps,
    ColumnPinnedProps,
    ColumnPivotProps,
    ColumnRenderingProps,
    ColumnRowDragProps,
    ColumnRowGroupProps,
    ColumnSortProps,
    ColumnSpanProps,
    ColumnTooltipProps,
    ColumnWidthProps,
    AgGridResourceBase,
):
    """Column definition for ag-grid."""

    @classmethod
    def create(cls, known_components: list[str], **props):
        """Create a ColumnDef with known components for component resolution."""
        instance = cls(**props)
        instance._known_components = known_components  # pyright: ignore [reportAttributeAccessIssue]
        return instance


class ColumnGroupBaseProps(PropsBase):
    """Column group base properties for ag-grid."""

    # A list containing a mix of columns and column groups.
    children: (
        list["ColumnDef | ColumnGroupDef"] | Var[list["ColumnDef | ColumnGroupDef"]]
    )

    # The unique ID to give the column. This is optional. If missing, a unique ID will be generated. This ID is used to identify the column group in the API.
    group_id: str | Var[str] | None = None

    # Set to true to keep columns in this group beside each other in the grid. Moving the columns outside of the group (and hence breaking the group) is not allowed.
    marry_children: bool | Var[bool] | None = None

    # Set to true if this group should be opened by default.
    open_by_default: bool | Var[bool] | None = None

    # Whether to only show the column when the group is open / closed. If not set the column is always displayed as part of the group.
    column_group_show: Literal["open", "closed"] | Var[str] | None = None

    # CSS class to use for the tool panel cell. Can be a string, array of strings, or function.
    tool_panel_class: str | list[str] | Var[str] | Var[list[str]] | None = None

    # Set to true if you do not want this column or group to appear in the Columns Tool Panel.
    suppress_columns_tool_panel: bool | Var[bool] | None = None

    # Set to true if you do not want this column (filter) or group (filter group) to appear in the Filters Tool Panel.
    suppress_filters_tool_panel: bool | Var[bool] | None = None

    # Provide your own tooltip component for the column group.
    tooltip_component: Any | None = None

    # Params to be passed to the tooltipComponent.
    tooltip_component_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Context property that can be used to associate arbitrary application data with this column definition.
    context: Any | None = None


class ColumnGroupHeaderProps(PropsBase):
    """Column group header properties for ag-grid."""

    # The name to render in the column header. If not specified and field is specified, the field name will be used as the header name.
    header_name: str | Var[str] | None = None

    # CSS class to use for the header cell. Can be a string, array of strings, or function.
    header_class: str | list[str] | Var[str] | Var[list[str]] | None = None

    # Tooltip for the column header
    header_tooltip: str | Var[str] | None = None

    # If enabled then the column header row will automatically adjust height to accommodate the size of the header cell. This can be useful when using your own headerComponent or long header names in conjunction with wrapHeaderText.
    auto_header_height: bool | None = None

    # The custom header group component to be used for rendering the component header. If none specified the default AG Grid is used.
    header_group_component: Any | None = None

    # The params used to configure the headerGroupComponent.
    header_group_component_params: dict[str, Any] | Var[dict[str, Any]] | None = None

    # Set to true if you don't want the column header for this column to span the whole height of the header container.
    suppress_span_header_height: bool | None = None

    # If true the label of the Column Group will not scroll alongside the grid to always remain visible.
    suppress_sticky_label: bool | None = None


class ColumnGroupDef(
    ColumnGroupHeaderProps,
    ColumnGroupBaseProps,
    AgGridResourceBase,
):
    """Column group definition for ag-grid."""

    @classmethod
    def create(cls, known_components: list[str], **props):
        """Create a ColumnGroupDef with known components for component resolution."""
        instance = cls(**props)
        instance._known_components = known_components  # pyright: ignore [reportAttributeAccessIssue]
        return instance

    def dict(self, *args, **kwargs):
        """Override dict method to exclude None values."""
        kwargs.setdefault("exclude_none", True)

        return super().dict(*args, **kwargs)



"""Autocompletes input with dropdown."""

import reflex as rx
from reflex.components.component import Component  # noqa: F401

from reflex_enterprise.components.mantine.base import (
    MantineDescriptionProps,
    MantineErrorProps,
    MantineLabelProps,
    MantineSize,
)
from reflex_enterprise.components.mantine.combobox import (
    Combobox,
    MiddlewaresProps,  # noqa: F401
)


class Autocomplete(
    MantineDescriptionProps, MantineErrorProps, MantineLabelProps, Combobox
):
    """Autocomplete input with dropdown."""

    tag = "Autocomplete"

    # Props passed down to the clear button
    clear_button_props: rx.Var[dict]

    # Determines whether the clear button should be displayed in the right section when the component has value, false by default
    clearable: rx.Var[bool]

    # Props passed down to Combobox component
    combobox_props: rx.Var[dict]

    # Data displayed in the dropdown. Values must be unique, otherwise an error will be thrown and component will not render.
    data: rx.Var[list[str]]

    # Uncontrolled dropdown initial opened state
    default_dropdown_opened: rx.Var[bool]

    # Uncontrolled component default value
    default_value: rx.Var[list[str]]

    # Sets disabled attribute on the input element
    disabled: rx.Var[bool]

    # Controlled dropdown opened state
    dropdown_opened: rx.Var[bool]

    # max-height of the dropdown, only applicable when withScrollArea prop is true, 250 by default
    dropdown_max_height: rx.Var[float | str]

    # Controls input height and horizontal padding, 'sm' by default
    size: rx.Var[MantineSize | str]

    # Value of the input
    value: rx.Var[str]

    # Event handler for dropdown close
    on_dropdown_close: rx.EventHandler[rx.event.no_args_event_spec]

    # Event handler for dropdown open
    on_dropdown_open: rx.EventHandler[rx.event.no_args_event_spec]

    # Event handler when an option is submitted, receives item value
    on_option_submit: rx.EventHandler[lambda item: [item]]


autocomplete = Autocomplete.create



"""AG Grid with custom formatters."""

import datetime
from typing import Any, TypedDict

import reflex as rx
from reflex.components import dynamic

import reflex_enterprise as rxe
from reflex_enterprise.components.ag_grid.resource import RendererParams

from .common import demo

# This ensures that `@rx.memo` components are available to cell_renderer functions.
dynamic.bundle_library("$/utils/components")

# A regex defined as literal JS expression
GROUP_THREE_DIGITS = rx.vars.StringVar(r"/\B(?=(\d{3})+(?!\d))/g")


class RowData(TypedDict):
    """Type for the row data."""

    name: str
    percent: float
    country: str
    number: float
    symbol: str


# The row data used by this example.
row_data: list[RowData] = [
    {
        "name": "John",
        "percent": 0.56,
        "country": "USA",
        "number": 12345.6789,
        "symbol": "$",
    },
    {
        "name": "Anna",
        "percent": 0.25,
        "country": "Sweden",
        "number": 3243.6789,
        "symbol": "€",
    },
    {
        "name": "Tom",
        "percent": 0.12,
        "country": "Germany",
        "number": 12345.6789,
        "symbol": "£",
    },
    {
        "name": "Mike",
        "percent": 0.35,
        "country": "Canada",
        "number": 8745.1234,
        "symbol": "$",
    },
    {
        "name": "Chaz",
        "percent": 0.9999,
        "country": "Utopia",
        "number": 42.42,
        "symbol": "฿",
    },
]

# A formatter function can be defined in Javascript as a FunctionStringVar
PERCENT_FORMATTER = rx.vars.FunctionStringVar.create("""(params) => {
if (typeof params.value === 'undefined' || Number.isNaN(params.value)) {
    return '';
}
var rounded = (params.value * 100).toFixed(2);
return `${rounded}%`;}
""")


# Alternatively, a formatter can be defined as a Python function that accepts
# Var typed arguments. The following type is defined to ensure that Var
# Operations use the correct field types for the row automatically.
class FormatterParams(rx.Base):
    """Type for the formatter params."""

    data: RowData
    value: str | float


# These python formatter functions accept Var typed arguments and may only use
# Var Operations to construct the return value.
def currency_formatter(params: rx.vars.ObjectVar[FormatterParams]) -> str:
    """Format a number as currency."""
    rounded_value = round(params.data.number, 2).to_string()
    money_value = rounded_value.replace(GROUP_THREE_DIGITS, ",")
    return f"{params.data.symbol}{money_value}"


def flag_formatter(params: rx.vars.ObjectVar[FormatterParams]) -> rx.Var[str]:
    """Format a country as a flag emoji."""
    return rx.Var.create(
        {
            "USA": "🇺🇸",
            "Sweden": "🇸🇪",
            "Germany": "🇩🇪",
            "Canada": "🇨🇦",
        }
    ).get(params.value.to(str), "🏳️‍🌈")  # Default to rainbow flag if country not found


cols_defs: list[dict] = [
    {"field": "name"},
    {
        "field": "name reversed",
        # synthetic column getter defined as a string (plotly style).
        "value_getter": {"function": 'params.data.name.split("").reverse().join("")'},
    },
    # Formatter defined as a plain string, interpreted as a JS expression with `params` in scope.
    {"field": "country", "value_formatter": "params.value.toUpperCase()"},
    # Formatter defined as FunctionStringVar.
    {"field": "percent", "value_formatter": PERCENT_FORMATTER},
    {
        "field": "flag",
        # Synthetic column getter defined as a python lambda.
        "value_getter": lambda params: params.data.country,
        # Formatter defined as a python function of Var typed arguments.
        "value_formatter": flag_formatter,
    },
    {
        "field": "number",
        # Cell renderer defined as a python lambda with generic params.
        "cell_renderer": lambda params: rx.text(
            params.value,
            font_family="monospace",
            line_height="inherit",
            color="rebeccapurple",
        ),
    },
    {
        "field": "currency number",
        "value_getter": "params.data.number",
        # Formatter defined as a python function of Var typed arguments.
        "value_formatter": currency_formatter,
    },
    {
        "field": "scaled number",
        # Synthetic column getter defined as a plain string, interpreted as a JS expression with `params` in scope.
        "value_getter": "params.data.number * params.data.percent",
        # Formatter defined as a lambda with generic params (why .to(float) is needed).
        "value_formatter": lambda params: round(params.value.to(float), 4),
        # Renderer defined as a lambda with generic params.
        "cell_renderer": lambda params: rx.tooltip(
            rx.text(params.valueFormatted, line_height="inherit", width="fit-content"),
            content=f"{params.data.number} * {params.data.percent}",
            side="left",
        ),
    },
    {
        "field": "row counter",
        "header_name": "# Clicks (Last click)",
        # Cell renderer is complex, so use `@rx.memo` component to encapsulate it.
        "cell_renderer": lambda params: row_counter(rowid=params.node.id),
        "sortable": False,
    },
    {
        "field": "raw data",
        "header_name": "Show raw data",
        # The component was registered with the grid and can be passed by name.
        "cell_renderer": "dataDialog",
        "sortable": False,
    },
]


# The following state and rx.memo component shows how to include complex interactive
# elements in the grid using a cell_renderer (see "row counter" field).
class RowClickCounterState(rx.State):
    """Keep track of button clicks per row."""

    row_clicks: dict[str, int] = {}
    last_click: dict[str, datetime.datetime] = {}

    @rx.event
    def handle_click(self, rowid: str):
        """Handle a row click."""
        if rowid in self.row_clicks:
            self.row_clicks[rowid] += 1
        else:
            self.row_clicks[rowid] = 1
        self.last_click[rowid] = datetime.datetime.now()

    @rx.var
    def button_colors(self) -> dict[str, str]:
        """Get the button colors."""
        return {
            rowid: "red" if count % 2 == 0 else "blue"
            for rowid, count in self.row_clicks.items()
        }


# Because react hooks cannot be evaluated at runtime, it is not possible to
# directly return dynamic components (like rx.moment) or use component event
# triggers in a renderer function. Instead, wrapping the component in rx.memo
# enables the hooks/imports/custom code to be evaluated at compile time and the
# resulting component may then referenced from an eval'd renderer function.
@rx.memo
def row_counter(rowid: str) -> rx.Component:
    """Create a row counter component."""
    return rx.flex(
        rx.button(
            RowClickCounterState.row_clicks.get(rowid, 0),
            rx.cond(
                RowClickCounterState.last_click[rowid],
                rx.moment(
                    RowClickCounterState.last_click[rowid],
                    format="(mm:ss)",
                    interval=1000,
                    duration_from_now=True,
                ),
                " (∞)",
            ),
            color_scheme=RowClickCounterState.button_colors.get(rowid, "gray"),
            on_click=RowClickCounterState.handle_click(rowid),
        ),
        height="100%",
        align="center",
    )


# Alternatively, a component can be registered with the grid at compile time and
# then set by string in the column def. Use rxe.static decorator to allow hooks,
# but preclude direct usage from state. This is fine, because in state we
# reference it by the registered name.
@rxe.static
def data_dialog(params: rx.vars.ObjectVar[RendererParams]) -> rx.Component:
    """Create a data dialog component."""
    return rx.flex(
        rx.dialog.root(
            rx.dialog.trigger(rx.button("Raw Data")),
            rx.dialog.content(
                rx.dialog.title("Raw Row Data"),
                rx.dialog.description("JSON representation of the row data."),
                rx.code_block(
                    params.data.to_string(),
                ),
            ),
        ),
        height="100%",
        align="center",
    )


class FormatterState(rx.State):
    """State for the formatter demo."""

    cols_defs: list[dict] = cols_defs


def formatter_grid(id: str, column_defs: Any) -> rx.Component:
    """Create a grid with the given id, column defs and default props."""
    return rxe.ag_grid(
        id=id,
        column_defs=column_defs,
        row_data=row_data,
        width="100%",
        height="50vh",
        auto_size_strategy={"type": "fitCellContents"},
        components={
            "dataDialog": data_dialog,
        },
    )


@demo(
    route="/formatters",
    title="AG Grid Formatters",
    description="AG Grid with custom formatters.",
)
def formatter_page():
    """AG Grid with custom formatters."""
    return rx.vstack(
        rx.box(
            row_counter(rowid=""), display="none"
        ),  # to ensure row_counter is compiled.
        rx.text(
            "Each of the three methods of setting column defs should work the same"
        ),
        rx.tabs.root(
            rx.tabs.list(
                rx.tabs.trigger(
                    "Inline",
                    value="inline",
                ),
                rx.tabs.trigger(
                    "State",
                    value="state",
                ),
                rx.tabs.trigger(
                    "API",
                    value="api",
                ),
            ),
            rx.tabs.content(
                formatter_grid(
                    id="formatter-grid-bare",
                    column_defs=cols_defs,
                ),
                value="inline",
            ),
            rx.tabs.content(
                formatter_grid(
                    id="formatter-grid-state",
                    column_defs=FormatterState.cols_defs,
                ),
                value="state",
            ),
            rx.tabs.content(
                rx.button(
                    "Set column defs",
                    on_click=[
                        (api := rxe.ag_grid.api("formatter-grid-api")).set_grid_option(
                            "column_defs",
                            cols_defs,
                        ),
                        api.size_columns_to_fit(),
                    ],
                ),
                rx.button(
                    "Clear column defs",
                    on_click=rxe.ag_grid.api("formatter-grid-api").set_grid_option(
                        "columnDefs",
                        [],
                    ),
                ),
                formatter_grid(id="formatter-grid-api", column_defs=[]),
                value="api",
            ),
            default_value="inline",
            width="100%",
        ),
        width="100%",
    )



"""OIDC helpers: metadata, JWKS, and JWT verification."""

import base64
import hashlib
import logging

import httpx
from joserfc import jwk, jwt

# Simple cached OIDC metadata + JWKS loader
_OIDC_CACHE: dict[str, dict] = {}


logger = logging.getLogger(__name__)


class ValidationError(ValueError):
    """Raised when token validation fails."""


async def _fetch_oidc_metadata(issuer: str) -> dict:
    key = f"metadata:{issuer}"
    if key in _OIDC_CACHE:
        return _OIDC_CACHE[key]
    url = issuer.rstrip("/") + "/.well-known/openid-configuration"
    async with httpx.AsyncClient() as client:
        resp = await client.get(url, timeout=10)
        resp.raise_for_status()
        md = resp.json()
        _OIDC_CACHE[key] = md
        return md


async def _fetch_jwks(jwks_uri: str) -> dict:
    key = f"jwks:{jwks_uri}"
    if key in _OIDC_CACHE:
        return _OIDC_CACHE[key]
    async with httpx.AsyncClient() as client:
        resp = await client.get(jwks_uri, timeout=10)
        resp.raise_for_status()
        jwks = resp.json()
        _OIDC_CACHE[key] = jwks
        return jwks


async def issuer_endpoint(issuer_uri: str, service: str) -> str:
    """Fetch an endpoint URL (authorization/token/userinfo/etc) from OIDC metadata."""
    try:
        return (await _fetch_oidc_metadata(issuer_uri))[service]
    except KeyError:
        return ""


async def verify_jwt(
    token_json: str,
    issuer: str,
    audience: str | list[str] | None = None,
    nonce: str | None = None,
    at_hash: str | None = None,
) -> jwt.Token:
    """Verify a JWT using OIDC discovery and JWKS per Microsoft docs.

    Returns:
        The verified JWT token.
    """
    # Fetch metadata and keys from well-known endpoint.
    md = await _fetch_oidc_metadata(issuer)
    jwks_uri = md.get("jwks_uri")
    if not jwks_uri:
        raise RuntimeError("jwks_uri not found in oidc metadata")
    jwks = await _fetch_jwks(jwks_uri)

    # Build JsonWebKey set from HTTP response/cache.
    key_set = jwk.KeySet.import_key_set(jwks)  # pyright: ignore[reportArgumentType]

    # Decode and validate the JWT signature.
    token = jwt.decode(token_json, key_set)

    # Validate specific claims, if provided.
    iss_claim_request: jwt.ClaimsOption = {"essential": True, "value": issuer}
    aud_claim_request: jwt.ClaimsOption = {"essential": False}
    if audience is not None:
        aud_claim_request["essential"] = True
        if isinstance(audience, str):
            aud_claim_request["value"] = audience
        else:
            aud_claim_request["values"] = audience

    nonce_claim_request: jwt.ClaimsOption = {"essential": False}
    if nonce is not None:
        nonce_claim_request["essential"] = True
        nonce_claim_request["value"] = nonce

    at_hash_claim_request: jwt.ClaimsOption = {"essential": False}
    if at_hash:
        at_hash_claim_request["value"] = at_hash

    logger.debug(
        f"Verifying JWT claims: iss={iss_claim_request}, aud={aud_claim_request}, nonce={nonce_claim_request}, at_hash={at_hash_claim_request}"
    )

    claims_requests = jwt.JWTClaimsRegistry(
        iss=iss_claim_request,
        aud=aud_claim_request,
        nonce=nonce_claim_request,
        at_hash=at_hash_claim_request,
    )
    claims_requests.validate(token.claims)
    return token


def compute_at_hash(access_token: str, alg: str | None) -> str:
    """Compute at_hash value per OIDC spec for a given access token and alg."""
    if not alg:
        alg = "RS256"
    # map alg to hash
    if alg.endswith("256"):
        h = hashlib.sha256(access_token.encode("utf-8")).digest()
    elif alg.endswith("384"):
        h = hashlib.sha384(access_token.encode("utf-8")).digest()
    elif alg.endswith("512"):
        h = hashlib.sha512(access_token.encode("utf-8")).digest()
    else:
        h = hashlib.sha256(access_token.encode("utf-8")).digest()
    # left-most half
    half = h[: len(h) // 2]
    at_hash = base64.urlsafe_b64encode(half).decode("ascii").rstrip("=")
    return at_hash



"""Demo of a customized ModelWrapper with auth state."""

import asyncio
from typing import Any

import reflex as rx

import reflex_enterprise as rxe
from reflex_enterprise.components.ag_grid.datasource import SSRMDatasourceRequestParams

from .common import demo
from .model_wrapper_customized import AuthState
from .model_wrapper_simple import Friend


class CustomDatasourceParams(SSRMDatasourceRequestParams):
    """Custom request parameters for SSRM."""

    sid: str


# When extending a ModelWrapper, you can override methods to customize the behavior.
class FriendModelWrapperSSRM(rxe.ModelWrapperSSRM[Friend]):
    """Customized ModelWrapper for the Friend model."""

    __get_data_kwargs__ = {
        **rxe.ModelWrapperSSRM.__get_data_kwargs__,
        "sid": lambda self: self.router.session.session_id,
    }
    __data_source_params_class__ = CustomDatasourceParams

    enable_advanced_filter: bool = False

    def _get_column_defs(self):
        """In this example, we remove the ability to filter and sort a particular field."""
        cols = super()._get_column_defs()
        for col in cols:
            if col.field == "spouse_is_annoying":
                col.filter = None
                col.sortable = False
        return cols

    @rx.event
    async def on_value_setter(
        self, row_data: dict[str, Any], field_name: str, value: Any
    ):
        """In this example, we prevent modifications to the data if the user is not logged in."""
        auth_state = await self.get_state(AuthState)
        if not auth_state.logged_in:
            return  # no modification for logged out users
        return await super().on_value_setter(row_data, field_name, value)

    async def _get_data(
        self,
        params: CustomDatasourceParams,
    ):
        print(f"_get_data: CustomDatasourceParams.sid = {params.sid}")  # noqa: T201
        auth_state = await self.get_state(AuthState)
        if not auth_state.logged_in:
            return []  # no records for logged out users
        await asyncio.sleep(0.2)
        return await super()._get_data(params)

    @rx.var
    def selected_items(self) -> list[Friend]:
        """Get the selected items from the grid."""
        # Normally selected items are backend-only, but we can provide
        # a computed var to render them in the UI.
        return self._selected_items

    @rx.event
    def set_enable_advanced_filter(self, v: bool):
        """Set whether to use the advanced filter."""
        self.enable_advanced_filter = v

    @classmethod
    def _default_grid_props(cls) -> dict[str, Any]:
        return {
            **super()._default_grid_props(),
            "enable_advanced_filter": cls.enable_advanced_filter,
        }


# Advanced example of an extended ModelWrapper with custom behavior
@demo(
    route="/model-ssrm",
    title="SSRM ModelWrapper",
    description="Extended SSRM ModelWrapper with custom behavior and auth.",
)
def model_page_ssrm():
    """Page for the customized ModelWrapper demo."""
    grid = FriendModelWrapperSSRM.create(
        model_class=Friend,
        row_selection={"mode": "multiRow"},
        on_filter_opened=rx.console_log,
        on_filter_changed=rx.console_log,
        on_filter_modified=rx.console_log,
        on_filter_ui_changed=rx.console_log,
        on_floating_filter_ui_changed=rx.console_log,
        on_advanced_filter_builder_visible_changed=rx.console_log,
        loading_cell_renderer=rx.vars.function.ArgsFunctionOperation.create(
            args_names=["params"],
            return_expr=rx.Var.create(
                rx.box(
                    rx.cond(
                        rx.Var("params.node.failedLoad"),
                        rx.hstack(
                            rx.icon("x"),
                            rx.text(
                                "Failed to load rows: ",
                                rx.Var("params.node.parent.failReason"),
                                align="center",
                            ),
                        ),
                        rx.hstack(
                            rx.spinner(), rx.text("Loading rows..."), align="center"
                        ),
                    ),
                    padding_x="5px",
                ),
            ),
        ),
    )
    return rx.vstack(
        rx.hstack(
            rx.cond(
                AuthState.logged_in,
                rx.button("Logout", on_click=AuthState.toggle_login),
                rx.button("Login", on_click=AuthState.toggle_login),
            ),
            rx.cond(
                AuthState.logged_in,
                rx.button("Generate Friends", on_click=AuthState.generate_friends(50)),
            ),
            rx.foreach(
                grid.State.selected_items,  # pyright: ignore [reportAttributeAccessIssue]
                lambda friend: rx.badge(friend.name),
            )
            if grid.State
            else (),
            rx.spacer(),
            rx.card(
                rx.hstack(
                    "Enable Advanced Filter",
                    rx.switch(
                        checked=grid.State.enable_advanced_filter,
                        on_change=grid.State.set_enable_advanced_filter,
                    ),
                ),
            ),
            align="center",
            width="100%",
        ),
        rx.box(
            grid,
            width="100%",
            height="65vh",
            padding_bottom="60px",  # for scroll bar and controls
        ),
    )



"""NumberFormatter component for formatting numbers in a user-friendly way."""

from typing import Literal

import reflex as rx

from reflex_enterprise.components.mantine.base import MantineCoreBase


class NumberFormatter(MantineCoreBase):
    """Formats number input/output."""

    tag = "NumberFormatter"

    # If true, negative numbers are allowed.
    allow_negative: rx.Var[bool]

    # Limits the number of digits after the decimal separator.
    decimal_scale: rx.Var[int]

    # Character used as decimal separator.
    decimal_separator: rx.Var[str]

    # If true, the number will be padded with zeros to match the decimal scale.
    fixed_decimal_scale: rx.Var[bool]

    # Adds prefix to the formatted value, eg. '$ '.
    prefix: rx.Var[str]

    # Adds suffix to the formatted value, eg. ' %'.
    suffix: rx.Var[str]

    # Character used as thousand separator. Set to true to automatically determine the separator based on the user's locale.
    thousand_separator: rx.Var[str | bool]

    # Defines the thousand grouping style.
    thousands_group_style: rx.Var[Literal["none", "thousand", "lakh", "wan"]]

    # The value to be formatted.
    value: rx.Var[str | int | float]


number_formatter = NumberFormatter.create



"""Pivot demo for AgGrid."""

import pandas as pd

import reflex_enterprise as rxe

from .common import demo

df = pd.read_json("https://www.ag-grid.com/example-assets/olympic-winners.json")


@demo(
    route="/pivot",
    title="Pivot",
    description="AgGrid with pivoting",
)
def pivot_page():
    """¨Pivot demo."""
    return rxe.ag_grid(
        id="sandbox_grid",
        column_defs=[
            {"field": "country", "row_group": True},
            {"field": "sport", "pivot": True},
            {"field": "year", "pivot": True},
            {"field": "gold", "aggFunc": "sum"},
        ],
        loading=False,
        row_data=df.to_dict("records"),  # pyright: ignore [reportArgumentType]
        default_col_def={
            "flex": 1,
            "min_width": 130,
            "enable_value": True,
            "enable_row_group": True,
            "enable_pivot": True,
        },
        auto_group_column_def={
            "minWidth": 200,
            "pinned": "left",
        },
        pivot_mode=True,
        side_bar="columns",
        pivot_panel_show="always",
        width="100%",
        height="500px",
    )



"""MultiSelect component for Reflex Enterprise."""

from typing import Any, List, Literal

import reflex as rx
from reflex.components.component import Component

from reflex_enterprise.components.mantine.base import BaseMantineInput


class MultiSelect(BaseMantineInput):
    """Renders a multiselect component."""

    tag = "MultiSelect"

    # If set, value can be cleared by clicking the clear button located on the right side of the input, by default hidden
    clearable: rx.Var[bool]

    # Props passed down to the clear button
    clear_button_props: rx.Var[dict]

    # Props passed down to the underlying Combobox component
    combobox_props: rx.Var[dict]

    # Data used to render options. Can be an array of strings or objects with value and label properties.
    data: rx.Var[List[str | dict]]

    # Initial value for uncontrolled component
    default_value: rx.Var[List[str]]

    # Input description, displayed below the input
    description: rx.Var[str | Component]

    # Props passed down to the description element
    description_props: rx.Var[dict]

    # If true, the input is disabled
    disabled: rx.Var[bool]

    # Controlled dropdown opened state
    dropdown_opened: rx.Var[bool]

    # Input error message, displayed below the input
    error: rx.Var[str | Component]

    # Props passed down to the error message element
    error_props: rx.Var[dict]

    # Function based on which items are filtered and sorted
    filter: rx.Var[Any]

    # Determines whether picked options should be removed from the dropdown. False by default.
    hide_picked_options: rx.Var[bool]

    # Function to customize the input container element.
    input_container: rx.Var[Any]

    # Controls the order of elements within the input wrapper.
    input_wrapper_order: rx.Var[List[Literal["label", "description", "input", "error"]]]

    # Input label, displayed above the input
    label: rx.Var[str | Component]

    # Props passed down to the label element
    label_props: rx.Var[dict]

    # Maximum number of options displayed in the dropdown, Infinity by default
    limit: rx.Var[int]

    # Maximum dropdown height, defaults to `220`
    max_dropdown_height: rx.Var[str | float | int]

    # Maximum number of values that can be picked
    max_values: rx.Var[int]

    # Message displayed when no options match the search query
    nothing_found_message: rx.Var[str | Component]

    # Props passed down to the underlying PillsInput component
    pills_input_props: rx.Var[dict]

    # Placeholder displayed when the input is empty
    placeholder: rx.Var[str]

    # Determines whether the input should have `cursor: pointer` style, `false` by default
    pointer: rx.Var[bool]

    # Input border-radius, key of `theme.radius` or any valid CSS value. Defaults to `theme.defaultRadius`.
    radius: rx.Var[str | int | Literal["xs", "sm", "md", "lg", "xl"]]

    # Function to customize option rendering
    render_option: rx.Var[Any]

    # Determines whether the input is required, adds an asterisk `*` to the label if `withAsterisk` is not set
    required: rx.Var[bool]

    # Determines whether the options should be filtered based on the search query, `false` by default
    searchable: rx.Var[bool]

    # Controlled search value
    search_value: rx.Var[str]

    # Determines whether the first option should be selected when the search query changes, `false` by default
    select_first_option_on_search: rx.Var[bool]

    # Input size, key of `theme.spacing` or any valid CSS value. Defaults to `md`.
    size: rx.Var[str | Literal["xs", "sm", "md", "lg", "xl"]]

    # Controlled component value
    value: rx.Var[List[str]]

    # Input variant, defaults to `default`
    variant: rx.Var[str]

    # Determines whether the required asterisk `*` should be displayed next to the label. True by default.
    with_asterisk: rx.Var[bool]

    # Props passed down to the root element
    wrapper_props: rx.Var[dict]

    # Determines whether the error state styles should be applied. True by default.
    with_error_styles: rx.Var[bool]

    # Called when the value changes
    on_change: rx.EventHandler[lambda value: [value]]

    # Called when the dropdown is closed
    on_dropdown_close: rx.EventHandler

    # Called when the dropdown is opened
    on_dropdown_open: rx.EventHandler

    # Called when the search value changes
    on_search_value_change: rx.EventHandler[lambda value: [value]]

    # Called when an option is selected from the dropdown, not called when the value changes programmatically
    on_option_submit: rx.EventHandler[lambda value: [value]]


multi_select = MultiSelect.create



"""Type definitions for OIDC authentication in Reflex Enterprise."""

from typing import TypedDict


class OIDCUserInfo(TypedDict, total=False):
    """Type describing user info obtained from an OIDC provider.

    Attributes:
        sub: The unique identifier for the user.
    """

    sub: str



"""Datasource classes for AgGrid components."""

import json
import sys
from typing import Any, ClassVar, Generic, Sequence, Type, TypedDict, TypeVar

import reflex as rx
from reflex.base import Base
from reflex.components.props import PropsBase
from reflex.utils.format import to_camel_case
from reflex.utils.serializers import serialize
from reflex.vars import Var
from reflex.vars.function import ArgsFunctionOperation, FunctionStringVar, FunctionVar
from reflex.vars.object import ObjectVar
from reflex.vars.sequence import ArrayVar
from typing_extensions import NotRequired

from reflex_enterprise.components.ag_grid.handlers import M
from reflex_enterprise.utils import encode_uri_component, fetch, get_backend_url
from reflex_enterprise.vars import PromiseVar


def _try_json_loads(value: str) -> Any:
    """Try to load a JSON string.

    Args:
        value: The JSON string to load.

    Returns:
        The loaded JSON object or the original string if loading fails.
    """
    try:
        return json.loads(value)
    except ValueError:
        return value


class DatasourceBase(Base):
    """Base datasource class for AgGrid components."""

    rowCount: Var[int] | int | None = None  # noqa: N815
    getRows: Var | None = None  # noqa: N815

    uri: str | None = None
    endpoint_uri: str | Var[str] | None = None
    endpoint_kwargs: dict[str, str] | Var[dict[str, str]] | None = None

    def _get_rows_function(self) -> Var:
        if self.getRows is not None:
            return self.getRows
        raise NotImplementedError("_get_rows_function must be implemented by subclass")

    def dict(self, **kwargs):
        """Convert the object to a dictionary.

        Args:
            **kwargs: Arbitrary keyword arguments.

        Returns:
            The object as a dictionary.
        """
        kwargs.setdefault("exclude", set()).update(
            {"getRows", "uri", "endpoint_uri", "endpoint_kwargs"}
        )
        d = super().dict(**kwargs)
        d["getRows"] = self._get_rows_function()
        return d

    def json(self) -> str:
        """Convert the object to a json-like string.

        Vars will be unwrapped so they can represent actual JS var names and functions.

        Keys will be converted to camelCase.

        Returns:
            The object as a Javascript Object literal.
        """
        return self.__config__.json_dumps(
            {to_camel_case(key): value for key, value in self.dict().items()},
            default=serialize,
        )


DATASOURCE_PARAMS = TypeVar(
    "DATASOURCE_PARAMS",
    bound="DatasourceParamsBase",
)


class DatasourceParamsBase(PropsBase):
    """Base class for any Datasource params."""

    @classmethod
    def from_request(
        cls: Type[DATASOURCE_PARAMS], data: dict[str, str]
    ) -> DATASOURCE_PARAMS:
        """Create a DatasourceParams object from a dictionary.

        Args:
            data: The dictionary to convert.

        Returns:
            A DatasourceParams object.
        """
        kwargs = {
            to_camel_case(key): _try_json_loads(value) for key, value in data.items()
        }
        return cls(**kwargs)

    def __getattr__(self, item: str):
        """Get an attribute from the object."""
        if (cc_item := to_camel_case(item)) != item:
            return getattr(self, cc_item)
        return super().__getattribute__(item)


class DatasourceParams(DatasourceParamsBase):
    """Datasource getRows params for infinite row model."""

    startRow: int  # noqa: N815
    endRow: int  # noqa: N815
    sortModel: list[dict[str, str]]  # noqa: N815
    filterModel: dict[str, dict[str, str]]  # noqa: N815


class DatasourceEncodedVarMixin:
    """Common methods for Datasource param vars."""

    def __getattr__(self, item: str):
        """Get an attribute from the object."""
        if (cc_item := to_camel_case(item)) != item:
            return getattr(self, cc_item)
        return super().__getattr__(item)  # pyright: ignore [reportAttributeAccessIssue]

    def get_param_mapping(self) -> Var[dict[str, Any]]:
        """Get the mapping of params to their encoded values.

        Returns:
            A dictionary mapping param names to their encoded values.
        """
        return Var.create(
            {
                key: encode_uri_component(getattr(self, key).to_string())
                for key in self._var_type.__fields__
            }
        )


class DatasourceParamsVar(DatasourceEncodedVarMixin, ObjectVar[DatasourceParams]):
    """A Var wrapping DatasourceParams with some extra functionality."""

    @property
    def success_callback(self) -> FunctionVar:
        """Get the success callback function."""
        return FunctionStringVar.create(f"{self}.successCallback")

    @property
    def fail_callback(self) -> FunctionVar:
        """Get the fail callback function."""
        return FunctionStringVar.create(f"{self}.failCallback")


PARAMS = DatasourceParamsVar("params", _var_type=DatasourceParams)
DATA = ArrayVar("data", _var_type=list[dict[str, str]])


class Datasource(DatasourceBase):
    """Infinite row model datasource for AgGrid components."""

    __default_query_params__: ClassVar[Var[dict[str, Var]]] = PARAMS.get_param_mapping()

    def get_uri(self) -> Var[str] | str:
        """Get the URI for the datasource.

        Returns:
            The URI for the datasource.
        """
        if self.uri:
            return self.uri

        if not self.endpoint_uri:
            raise ValueError("uri or endpoint_uri must be set")

        query = (
            self.__default_query_params__.merge(self.endpoint_kwargs)  # pyright: ignore [reportAttributeAccessIssue]
            if self.endpoint_kwargs is not None
            else self.__default_query_params__
        )

        return Var.create(
            f"{self.endpoint_uri}?{query.items().foreach(lambda kv: f'{kv[0]}={kv[1]}').join('&')}"  # pyright: ignore [reportAttributeAccessIssue]
        )

    def _get_rows_function(self) -> str | Var:
        return ArgsFunctionOperation.create(
            args_names=(str(PARAMS),),
            return_expr=fetch(
                get_backend_url(self.get_uri()),
            )
            .then(
                ArgsFunctionOperation.create(
                    args_names=("response",),
                    return_expr=PromiseVar("response.json()").then(
                        ArgsFunctionOperation.create(
                            args_names=(str(DATA),),
                            return_expr=rx.cond(
                                DATA[1].js_type() == "number",
                                PARAMS.success_callback(
                                    DATA[0],
                                    DATA[1],
                                ),
                                PARAMS.success_callback(
                                    DATA,
                                    rx.cond(
                                        DATA.length()
                                        < (PARAMS.end_row - PARAMS.start_row),
                                        PARAMS.start_row + DATA.length(),
                                        -1,
                                    ),
                                ),
                            ),
                        )
                    ),
                )
            )
            .catch(
                ArgsFunctionOperation.create(
                    args_names=("error",), return_expr=PARAMS.fail_callback()
                )
            ),
        )


class SSRMDatasourceRequestParams(DatasourceParams):
    """Datasource getRows params for server-side row model."""

    rowGroupCols: list[dict[str, str]]  # noqa: N815
    groupKeys: list[str]  # noqa: N815
    valueCols: list[dict[str, str]]  # noqa: N815
    pivotMode: bool  # noqa: N815
    pivotCols: list[dict[str, str]]  # noqa: N815


class SSRMDatasourceParams(Base):
    """Datasource params for server-side row model."""

    request: SSRMDatasourceRequestParams


class SSRMDatasourceRequestParamsVar(
    DatasourceEncodedVarMixin, ObjectVar[SSRMDatasourceRequestParams]
):
    """A Var wrapping SSRMDatasourceRequestParams with some extra functionality."""


class SSRMDatasourceParamsVar(ObjectVar[SSRMDatasourceParams]):
    """A Var wrapping SSRMDatasourceParams with some extra functionality."""

    @property
    def request(self) -> SSRMDatasourceRequestParamsVar:
        """Get the request params."""
        return SSRMDatasourceRequestParamsVar(
            f"{self}.request", _var_type=SSRMDatasourceRequestParams
        )

    @property
    def success(self) -> FunctionVar:
        """Get the success callback function."""
        return FunctionStringVar.create(f"{self}.success")

    @property
    def fail(self) -> FunctionVar:
        """Get the fail callback function."""
        return FunctionStringVar.create(f"{self}.fail")


SSPARAMS = SSRMDatasourceParamsVar("params", _var_type=SSRMDatasourceParams)


if sys.version_info >= (3, 11):

    class LoadSuccessParams(TypedDict, Generic[M]):
        """Parameters that can be passed to the success callback of SSRM datasources."""

        rowData: Sequence[M]
        rowCount: NotRequired[int]
        groupLevelInfo: NotRequired[Any]
        pivotResultFields: NotRequired[Sequence[str]]
else:

    class LoadSuccessParams(TypedDict):
        """Parameters that can be passed to the success callback of SSRM datasources."""

        rowData: Sequence
        rowCount: NotRequired[int]
        groupLevelInfo: NotRequired[Any]
        pivotResultFields: NotRequired[Sequence[str]]


class SSRMDatasource(Datasource):
    """Server-side row model datasource class for AgGrid components."""

    __default_query_params__: ClassVar[Var[dict[str, Var]]] = (
        SSPARAMS.request.get_param_mapping()
    )

    def _get_rows_function(self) -> str | Var:
        return ArgsFunctionOperation.create(
            args_names=(str(SSPARAMS),),
            return_expr=fetch(
                get_backend_url(self.get_uri()),
            )
            .then(
                ArgsFunctionOperation.create(
                    args_names=("response",),
                    return_expr=PromiseVar("response.json()").then(
                        ArgsFunctionOperation.create(
                            args_names=(str(DATA),),
                            return_expr=SSPARAMS.success(
                                rx.cond(
                                    DATA.length().is_not_none(),
                                    {"rowData": DATA},
                                    DATA,
                                ),
                            ),
                        ),
                    ),
                ),
            )
            .catch(
                ArgsFunctionOperation.create(
                    args_names=("error",), return_expr=SSPARAMS.fail()
                )
            ),
        )
